#!/usr/bin/env python
# encoding: utf-8

"""
Created by Leonardo Ubeda on 14JUL2016
reads tables produced by
footprints_create_apertures-00.py
and then produces region files and displays them in DS9
Based on work by Colin Cox and Wayne Kinzel
Updated by Joseph Hunkeler 



version history
2.4      12MAR2018      changed from V3PA to aperture position angle
                        help desk ticket INC0048565

2.3      01JAN2018      added NIRSpec fixed slit apertures
                        added NIRCam 8NIRSPEC dither pattern
                        changed launch date and science cycles in timeline

2.2      14JUN2017      DS9 parameters in GUI
                        output directory
                        region files colours
2.0      06JUN2017      major update by Joseph Hunkeler
                        code ported to Python 3
                        compatible with Python 2.7
1.2      26MAY2017      added hhmmss format for RA 
                        added ddmmss format for DEC
                        changed the naming convention of dither patterns to FULL3 FULL3TIGHT FULL6
1.0      14JUL2016      first virsion

"""

from __future__ import absolute_import, division, print_function

import sys
import os
import numpy as np

from math import *
from astropy import wcs
from astropy.io import fits
from astropy.io import ascii
from astropy import units as u
from astropy.coordinates import SkyCoord
import pyds9
from . import PKG_DATA_DIR

#readfitsimage = True
#self.readfitsimageVar = StringVar()
#self.readfitsimageVar.set(self.config['readfitsimage'])
#print(readfitsimageVar)

def arcsec2deg(ra, dec, v2arcsec, v3arcsec, xr, yr):
    v2deg = ra + (v2arcsec - xr) / (3600. * cos(radians(dec)))
    v3deg = dec + ((v3arcsec - yr) / 3600.)
    return (v2deg, v3deg)


def unit(ra, dec):
    ''' Converts vector expressed in Euler angles to unit vector components.
    ra and dec in degrees
    Can be used for V2V3 after converting from arcsec to degrees)'''

    rar = radians(ra)
    decr = radians(dec)
    u = np.array([cos(rar) * cos(decr), sin(rar) * cos(decr), sin(decr)])
    return u


def radec(u):
    '''convert unit vector to Euler angles
    u is an array or list of length 3'''

    if len(u) != 3:
        print('Not a vector')
        return
    norm = sqrt(u[0]**2 + u[1]**2 + u[2]**2)  # Works for list or array
    dec = degrees(asin(u[2] / norm))
    ra = degrees(atan2(u[1], u[0]))  # atan2 puts it in the correct quadrant
    if ra < 0.0:
        ra += 360.0    # Astronomers prefer the range 0 to 360 degrees
    return (ra, dec)


def v2v3(u):
    '''Convert unit vector to v2v3'''
    if len(u) != 3:
        print('Not a vector')
        return
    norm = sqrt(u[0]**2 + u[1]**2 + u[2]**2)  # Works for list or array
    # atan2 puts it in the correct quadrant
    v2 = 3600 * degrees(atan2(u[1], u[0]))
    v3 = 3600 * degrees(asin(u[2] / norm))
    return (v2, v3)


def rotate(axis, angle):
    '''Fundamental rotation matrices.
    Rotate by angle measured in degrees, about axis 1 2 or 3'''
    if axis not in list(range(1, 4)):
        print('Axis must be in range 1 to 3')
        return
    r = np.zeros((3, 3))
    ax0 = axis - 1  # Allow for zero offset numbering
    theta = radians(angle)
    r[ax0, ax0] = 1.0
    ax1 = (ax0 + 1) % 3
    ax2 = (ax0 + 2) % 3
    r[ax1, ax1] = cos(theta)
    r[ax2, ax2] = cos(theta)
    r[ax1, ax2] = -sin(theta)
    r[ax2, ax1] = sin(theta)
    return r


def attitude(v2, v3, ra, dec, pa):
    '''This will make a rotation matrix which rotates a unit vector representing a v2,v3 position
    to a unit vector representing an RA, Dec pointing with an assigned position angle
    Described in JWST-STScI-001550, SM-12, section 6.1'''

    # v2, v3 in arcsec, ra, dec and position angle in degrees
    v2d = v2 / 3600.0
    v3d = v3 / 3600.0

    # Get separate rotation matrices
    mv2 = rotate(3, -v2d)
    mv3 = rotate(2, v3d)
    mra = rotate(3, ra)
    mdec = rotate(2, -dec)
    mpa = rotate(1, -pa)

    # Combine as mra*mdec*mpa*mv3*mv2
    m = np.dot(mv3, mv2)
    m = np.dot(mpa, m)
    m = np.dot(mdec, m)
    m = np.dot(mra, m)

    return m


def pointing(attitude, v2, v3):
    '''Using the attitude matrix to calculate where any v2v3 position points on the sky'''
    v2d = v2 / 3600.0
    v3d = v3 / 3600.0
    v = unit(v2d, v3d)
    w = np.dot(attitude, v)
    rd = radec(w)
    return rd  # tuple containing ra and dec


def linear_transformation(theta, xshift, yshift,
                          xscale, yscale, x2, x3, xr, yr):
    th = radians(theta)
    a = xshift
    d = yshift
    b = xscale * cos(th)
    c = yscale * sin(th)
    e = (-1) * xscale * sin(th)
    f = yscale * cos(th)
    v2r = a + b * (x2 - xr) + c * (x3 - yr)
    v3r = d + e * (x2 - xr) + f * (x3 - yr)
    return (v2r, v3r)


def rotate_long(x2, x3, theta):

    # search point in the center of each aperture
    xa = (x2[0] + x2[2]) / 2.0
    ya = (x3[0] + x3[2]) / 2.0
    xb = (x2[5] + x2[7]) / 2.0
    yb = (x3[5] + x3[7]) / 2.0
    # calculate center for rotation
    xr = ((xa + xb) / 2.)  # +xc
    yr = ((ya + yb) / 2.)  # +yc

    # rotate apertures according to the position angle PA
    rot = linear_transformation(theta, xr, yr, 1.0, 1.0, x2, x3, xr, yr)
    v2r = rot[0]
    v3r = rot[1]
    return (v2r, v3r, xr, yr)


def rotate_ifu(x2, x3, theta):
    # search point in the center of each aperture
    xr = (x2[0] + x2[2]) / 2.0
    yr = (x3[0] + x3[2]) / 2.0
    # rotate apertures according to the position angle PA
    rot = linear_transformation(theta, 0.0, 0.0, 1.0, 1.0, x2, x3, xr, yr)
    v2r = rot[0]
    v3r = rot[1]
    return (v2r, v3r, xr, yr)


def rotate_msa(x2, x3, theta):
    # search point in the center of each aperture
    xmsa1 = (x2[0] + x2[2]) / 2.0
    ymsa1 = (x3[0] + x3[2]) / 2.0
    xmsa2 = (x2[5] + x2[7]) / 2.0
    ymsa2 = (x3[5] + x3[7]) / 2.0
    xmsa3 = (x2[10] + x2[12]) / 2.0
    ymsa3 = (x3[10] + x3[12]) / 2.0
    xmsa4 = (x2[15] + x2[17]) / 2.0
    ymsa4 = (x3[15] + x3[17]) / 2.0

    # calculate center for rotation
    xr13 = (xmsa1 + xmsa3) / 2.0
    yr13 = (ymsa1 + ymsa3) / 2.0
    xr24 = (xmsa2 + xmsa4) / 2.0
    yr24 = (ymsa2 + ymsa4) / 2.0
    xr = (xr13 + xr24) / 2.
    yr = (yr13 + yr24) / 2.

    # rotate apertures according to the position angle PA
    rot = linear_transformation(theta, xr, yr, 1.0, 1.0, x2, x3, xr, yr)
    v2r = rot[0]
    v3r = rot[1]
    return (v2r, v3r, xr, yr)


def rotate_short(x2, x3, theta):
    # search point in the center of each aperture
    xsh1 = (x2[0] + x2[2]) / 2.0
    ysh1 = (x3[0] + x3[2]) / 2.0

    xsh2 = (x2[5] + x2[7]) / 2.0
    ysh2 = (x3[5] + x3[7]) / 2.0

    xsh3 = (x2[20] + x2[22]) / 2.0
    ysh3 = (x3[20] + x3[22]) / 2.0

    xsh4 = (x2[25] + x2[27]) / 2.0
    ysh4 = (x3[25] + x3[27]) / 2.0

    # calculate center for rotation
    xr13 = (xsh1 + xsh3) / 2.0
    yr13 = (ysh1 + ysh3) / 2.0
    xr24 = (xsh2 + xsh4) / 2.0
    yr24 = (ysh2 + ysh4) / 2.0
    xr = (xr13 + xr24) / 2.
    yr = (yr13 + yr24) / 2.

    # rotate apertures according to the position angle PA
    rot = linear_transformation(theta, xr, yr, 1.0, 1.0, x2, x3, xr, yr)
    v2r = rot[0]
    v3r = rot[1]
    return (v2r, v3r, xr, yr)

#------------------------------


def create_footprint(inputfile, ra, dec, napertures, footprintname, color):
    global w
    # input

    # ra = ra in degrees
    # dec = dec in degrees
    # footprintname is the name of the output file
    # napertures = number of apertures in footprint ( Nircam LONG = 2, NIRCam
    # short = 8, MSA = 4)

    nrows = napertures * 5
    #print(nrows)
    ra = np.array(ra, np.float_)  # np.array(ra, np.float_)
    dec = np.array(dec, np.float_)  # np.array(dec, np.float_)
    world = []
    # print ra
    #sys.exit("quitting now...")

    for i in range(0, nrows):
        #world = np.append(world,(ra[i],dec[i]))
        world.append((ra[i], dec[i]))
    pixcrd2 = w.wcs_world2pix(world, 1)
    #print(pixcrd2)
    xx = []
    yy = []
    for i in range(0, nrows):
        xx.append(pixcrd2[[i], 0])
        yy.append(pixcrd2[[i], 1])
        #xx = np.append(xx,pixcrd2[[i],0])
        #yy = np.append(yy,pixcrd2[[i],1])

    xwcs = np.array(xx, np.float_)
    ywcs = np.array(yy, np.float_)
    x = [float(i) for i in xwcs]
    y = [float(i) for i in ywcs]
    # print x
    # polygon x1 y1 x2 y2 x3 y3 ...

    outputfile = footprintname
    file = open(outputfile, "w")
    file.write(
        'global color=' +
        color +
        ' width=1 font="helvetica 15 normal roman"   select=0 highlite=1 \n')
    file.write('image\n')
    pos = 0
    for i in range(0, napertures):
        newline = 'polygon ' + str(x[pos]) + '  ' + str(y[pos]) + '  ' + str(x[pos + 1]) + '  ' + str(y[pos + 1]) + '  ' + str(x[pos + 2]) + '  ' + str(
            y[pos + 2]) + '  ' + str(x[pos + 3]) + '  ' + str(y[pos + 3]) + '  ' + str(x[pos + 4]) + '  ' + str(y[pos + 4]) + '  ' + '# text={}' + '\n'
        file.write(newline)
        pos = pos + 5
    file.close()
#------------------------------


#------------------------------
def create_footprint_center(inputfile, ra, dec, footprintname, color):
    global w

    '''
    # ra = ra in degrees
    # dec = dec in degrees
    # footprintname is the name of the output file
    # napertures = number of apertures in footprint ( Nircam LONG = 2, NIRCam
    # short = 8, MSA = 4)
    '''
    ra = np.array(ra, np.float_)
    dec = np.array(dec, np.float_)
    world = []
    world.append((ra, dec))
    pixcrd2 = w.wcs_world2pix(world, 1)
    xx = float(pixcrd2[[0], 0])
    yy = float(pixcrd2[[0], 1])
    c2 = "%10s" % str(xx)
    c3 = "%10s" % str(yy)

    # polygon x1 y1 x2 y2 x3 y3 ...

    outputfile = footprintname
    file = open(outputfile, "w")
    file.write(
        'global color=' +
        color +
        ' width=1 font="helvetica 15 normal roman"  select=0 highlite=1 \n')
    file.write('image\n')
    newline = 'point(' + c2 + ',' + c3 + ') # point=cross 20  \n'
    file.write(newline)
    file.close()

#------------------------------
def read_table(inputfile, delim=' '):
    records = []
    with open(inputfile, 'r') as fp:
        for row in fp:
            cols = [ x for x in row.strip().split(delim) if x ]
            records.append(cols)

    return list(zip(*records))


def footprints(inputfile,
               sourcelist,
               plot_long='No',
               plot_short='No',
               plot_msa='No',
               plot_sources='No',
               ra_long='202.47',
               dec_long='47.2',
               theta_long=0.0,
               dither_pattern_long='No',
               ra_msa='202.47',
               dec_msa='47.2',
               theta_msa=0.0,
               mosaic='No',
               usershiftv2=0.0,
               usershiftv3=0.0,
               colmsa='red',
               colshort='green',
               collong='blue',
               ds9cmap='grey',
               ds9limmin=0.0,
               ds9limmax=30.0,
               ds9scale='log',
               outdir='/Users/myname/Desktop/'):
               #readfitsimage=True):

    # verify that outdir exists
    if not os.path.exists(outdir):
        os.makedirs(outdir, mode=0o0755)
        print("creating directory "+ outdir )

    #print(outdir)
    global w

    usershiftv2 = np.array(usershiftv2, np.float_)
    usershiftv3 = np.array(usershiftv3, np.float_)

    # nircam long footprint
    ra_short = ra_long
    dec_short = dec_long
    theta_short = theta_long
    dither_pattern_short = dither_pattern_long

    # read image and its header
    # need to extend this to multi extension fits files
    # print(readfitsimage)
    # print('reading fits')
    hdulist = fits.open(inputfile)
    w = wcs.WCS(hdulist[0].header)    # assuming WCS is in extension 0
    '''
    if readfitsimage == True:
        print('reading fits')
        hdulist = fits.open(inputfile)
        w = wcs.WCS(hdulist[0].header)    # assuming WCS is in extension 0
        readfitsimage = False 
#        readfitsimage = False
    '''


    if plot_sources == 'Yes':
        print('creating region file from source list')
        # here we read the list ra dec and create a DS9 region file
        data = ascii.read(sourcelist)
        # this gives the number of columns in the input file
        #print(len(data.colnames))

        if len(data.colnames) == 2:
            flagsources = 2
            # in this case the user inputs ra dec
            ra = np.array(data['col1'], np.float_)
            dec = np.array(data['col2'], np.float_)
            #print(ra)
            #print(dec)
            world = []
            for i in range(len(ra)):
                world.append((ra[i], dec[i]))
            pixcrd2 = w.wcs_world2pix(world, 1)
            xx = []
            yy = []
            for i in range(len(ra)):
                xx.append(pixcrd2[[i], 0])
                yy.append(pixcrd2[[i], 1])
            xwcs = np.array(xx, np.float_)
            ywcs = np.array(yy, np.float_)
            x = [float(i) for i in xwcs]
            y = [float(i) for i in ywcs]

            outputfile = outdir+'/ds9-sources.reg'
            file = open(outputfile, "w")
            file.write(
                'global color=yellow width=1 font="helvetica 15 normal roman"   select=0 highlite=1 \n')
            file.write('image\n')
            #pos = 0
            for i in range(len(ra)):
                c2 = "%10s" % str(x[i])
                c3 = "%10s" % str(y[i])
                newline = 'circle(' + c2 + ',' + c3 + ',5) # text={}' + '\n'
                file.write(newline)
            file.close()

        if len(data.colnames) >= 3:
            flagsources = 3
            # in this case the user inputs ra dec source-type
            ra = np.array(data['col1'], np.float_)
            dec = np.array(data['col2'], np.float_)
            sourcetype = data['col3']
            #print(ra)
            #print(dec)
            #print(sourcetype.info)
            # select sources according to type
            a = np.where(sourcetype == 'F')[0]  # fillers
            rafill = ra[a]
            decfill = dec[a]
            a = np.where(sourcetype == 'P')[0]  # primary sources
            rap = ra[a]
            decp = dec[a]

            # region file of fillers
            world = []
            for i in range(len(rafill)):
                world.append((rafill[i], decfill[i]))
            pixcrd2 = w.wcs_world2pix(world, 1)
            xx = []
            yy = []
            for i in range(len(rafill)):
                xx.append(pixcrd2[[i], 0])
                yy.append(pixcrd2[[i], 1])
            xwcs = np.array(xx, np.float_)
            ywcs = np.array(yy, np.float_)
            x = [float(i) for i in xwcs]
            y = [float(i) for i in ywcs]

            outputfile = outdir+'/ds9-sources-fillers.reg'
            file = open(outputfile, "w")
            file.write(
                'global color=yellow width=1 font="helvetica 15 normal roman"   select=0 highlite=1 \n')
            file.write('image\n')
            #pos = 0
            for i in range(len(rafill)):
                c2 = "%10s" % str(x[i])
                c3 = "%10s" % str(y[i])
                newline = 'circle(' + c2 + ',' + c3 + ',5) # text={}' + '\n'
                file.write(newline)
            file.close()

            # region file of primary sources
            world = []
            for i in range(len(rap)):
                world.append((rap[i], decp[i]))
            pixcrd2 = w.wcs_world2pix(world, 1)
            xx = []
            yy = []
            for i in range(len(rap)):
                xx.append(pixcrd2[[i], 0])
                yy.append(pixcrd2[[i], 1])
            xwcs = np.array(xx, np.float_)
            ywcs = np.array(yy, np.float_)
            x = [float(i) for i in xwcs]
            y = [float(i) for i in ywcs]

            outputfile = outdir+'/ds9-sources-primary.reg'
            file = open(outputfile, "w")
            file.write(
                'global color=red width=1 font="helvetica 15 normal roman"  select=0  highlite=1 \n')
            file.write('image\n')
            #pos = 0
            for i in range(len(rap)):
                c2 = "%10s" % str(x[i])
                c3 = "%10s" % str(y[i])
                newline = 'circle(' + c2 + ',' + c3 + ',5) # text={}' + '\n'
                file.write(newline)
            file.close()
        if len(data.colnames) < 2:
            print('Invalid input file')
#-------------------------------------------------------------------
    #                                                                     nirspec msa
    #   15DEC2017  I add NIRSpec slits to be plotted with the MSA and IFU
    if plot_msa == 'Yes':
        print('processing NIRSPEC MSA')
        v2msa, v3msa, aper, v2ref, v3ref = read_table(os.path.join(PKG_DATA_DIR, 'table-nirspec-msa.txt'))
        v2msa = np.array(v2msa, np.float_)
        v3msa = np.array(v3msa, np.float_)

#---------------------------  
#  here i need to transform ra dec to float        06JUN2017  LEONARDO
    
        print('using NIRSpec RA  :',ra_msa)
        print('using NIRSpec DEC :',dec_msa)
        
        if (' ' in ra_msa) and (' ' in dec_msa) == True:   
           # it recognizes that the string has the format   hh mm ss.sss
           b = ra_msa+' '+dec_msa
           c = SkyCoord(b,unit=(u.hourangle, u.deg))
           #   transform to degrees to be used in the rest of the code
           ra = c.ra.deg 
           dec = c.dec.deg
           ra_msa = np.array(ra, np.float_)
           dec_msa = np.array(dec, np.float_)

        
        else:                          #   string is in units of degrees
           ra_msa = np.array(ra_msa, np.float_)
           dec_msa = np.array(dec_msa, np.float_)

#----------------------------------------------------------------------------  


        x2 = v2msa
        x3 = v3msa
        # search point in the center of each aperture
        xmsa1 = (x2[0] + x2[2]) / 2.0
        ymsa1 = (x3[0] + x3[2]) / 2.0
        xmsa2 = (x2[5] + x2[7]) / 2.0
        ymsa2 = (x3[5] + x3[7]) / 2.0
        xmsa3 = (x2[10] + x2[12]) / 2.0
        ymsa3 = (x3[10] + x3[12]) / 2.0
        xmsa4 = (x2[15] + x2[17]) / 2.0
        ymsa4 = (x3[15] + x3[17]) / 2.0

        # calculate center for rotation
        xr13 = (xmsa1 + xmsa3) / 2.0
        yr13 = (ymsa1 + ymsa3) / 2.0
        xr24 = (xmsa2 + xmsa4) / 2.0
        yr24 = (ymsa2 + ymsa4) / 2.0
        xr = (xr13 + xr24) / 2.
        yr = (yr13 + yr24) / 2.

        v2 = v2msa
        v3 = v3msa
        xr0 = xr
        yr0 = yr
        ra0 = ra_msa
        dec0 = dec_msa 
        pa = theta_msa - 137.4874   #  12MAR2018   NRS_FULL_MSA_V3IdlYang = 137.4874
        myv2 = []
        myv3 = []
        v20 = xr0
        v30 = yr0
        m = attitude(v20, v30, ra0, dec0, pa)
        for k in range(len(v2)):
            a = pointing(m, v2[k], v3[k])
            myv2 = np.append(myv2, a[0])
            myv3 = np.append(myv3, a[1])
        myv2 = np.array(myv2)
        myv3 = np.array(myv3)
        
        # create_footprint(input,myv2,myv3,5,'ds9-msa.reg','red') # 10 because
        # is includes the IFU aperture
        # 10 because it includes the IFU aperture and the slits
        create_footprint(
            input, 
            myv2, 
            myv3, 
            10,                                # here we have msa = 4 + ifu =1 + slits = 5 total =10 
            outdir+'/ds9-msa.reg', 
            colmsa)
        create_footprint_center(
            input,
            ra_msa,
            dec_msa,
            outdir+'/ds9-msa-centre.reg',
            colmsa)

    #-------------------------------------------------------------------
    # nircam long
    # print plot_long
    if plot_long == 'Yes':
        print('processing NIRCAM LWC')



#--------------------------------------------------------------------------  
#  here i need to transform ra dec to float             06JUN2017  LEONARDO
    
        print('using NIRCAm RA  :',ra_long)
        print('using NIRCam DEC :',dec_long)

        if (' ' in ra_long) and (' ' in dec_long) == True:   
           # it recognizes that the string has the format   hh mm ss.sss
           b = ra_long+' '+dec_long
           c = SkyCoord(b,unit=(u.hourangle, u.deg))
           #   transform to degrees to be used in the rest of the code
           ra = c.ra.deg 
           dec = c.dec.deg
           ra_long = np.array(ra, np.float_)
           dec_long = np.array(dec, np.float_)

        
        else:                          #   string is in units of degrees
           ra_long = np.array(ra_long, np.float_)
           dec_long = np.array(dec_long, np.float_)
#----------------------------------------------------------------------------  
        create_footprint_center(
            inputfile,
            ra_long,
            dec_long,
            outdir+'/ds9-long-centre.reg',
            collong)
        v2c, v3c, aper, v2ref, v3ref = read_table(os.path.join(PKG_DATA_DIR, 'table-nircam-long.txt'))
        v2_0 = np.array(v2c, np.float_)
        v3_0 = np.array(v3c, np.float_)

        if dither_pattern_long == 'None' and mosaic == 'No':
            #shiftv2 = [0.0, -58.0,  58.0]
            #shiftv3 = [0.0, -23.5,  23.5]
            v2 = v2_0
            v3 = v3_0
            # determine center of rotation
            xa = (v2[0] + v2[2]) / 2.0
            ya = (v3[0] + v3[2]) / 2.0
            xb = (v2[5] + v2[7]) / 2.0
            yb = (v3[5] + v3[7]) / 2.0
            xr = ((xa + xb) / 2.)
            yr = ((ya + yb) / 2.)
            xr0 = xr
            yr0 = yr
            ra0 = ra_long
            dec0 = dec_long
            pa = theta_long + 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            # for shft in (0,1,2):
            v20 = xr0
            v30 = yr0
            m = attitude(v20, v30, ra0, dec0, pa)
            for k in range(len(v2)):
                a = pointing(m, v2[k], v3[k])
                myv2 = np.append(myv2, a[0])
                myv3 = np.append(myv3, a[1])
            myv2 = np.array(myv2)
            myv3 = np.array(myv3)
            create_footprint(inputfile, 
               myv2,
               myv3, 
               2, 
               outdir+'/ds9-long-no.reg', 
               collong)

        #                                                                          8NIRSPEC  29NOV2017
        if dither_pattern_long == '8NIRSPEC':
            mosaic == 'No'
            shiftv2 = [-24.6, -24.4, 24.6, 24.4, 24.6, 24.4, -24.6, -24.4]
            shiftv3 = [-64.1, -89.0, -88.8, -63.9, 64.1, 89.0, 88.8, 63.9 ]
            v2 = v2_0
            v3 = v3_0
            # determine center of rotation
            xa = (v2[0] + v2[2]) / 2.0
            ya = (v3[0] + v3[2]) / 2.0
            xb = (v2[5] + v2[7]) / 2.0
            yb = (v3[5] + v3[7]) / 2.0
            xr = ((xa + xb) / 2.)
            yr = ((ya + yb) / 2.)
            xr0 = xr
            yr0 = yr
            ra0 = ra_long
            dec0 = dec_long
            pa = theta_long+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            #for shft in (0, 1, 2, 4, 5, 6, 7):
            for shft in range(len(shiftv2)):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            #print(len(myv2))    
            create_footprint(
                inputfile,
                myv2,
                myv3,
                16,
                outdir+'/ds9-long-8nirspec.reg',
                collong)


        if dither_pattern_long == 'FULL3' and mosaic == 'No':
            shiftv2 = [0.0, -58.0, 58.0]
            shiftv3 = [0.0, -23.5, 23.5]
            v2 = v2_0
            v3 = v3_0
            # determine center of rotation
            xa = (v2[0] + v2[2]) / 2.0
            ya = (v3[0] + v3[2]) / 2.0
            xb = (v2[5] + v2[7]) / 2.0
            yb = (v3[5] + v3[7]) / 2.0
            xr = ((xa + xb) / 2.)
            yr = ((ya + yb) / 2.)
            xr0 = xr
            yr0 = yr
            ra0 = ra_long
            dec0 = dec_long
            pa = theta_long+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            for shft in (0, 1, 2):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            create_footprint(
                inputfile,
                myv2,
                myv3,
                6,
                outdir+'/ds9-long-three.reg',
                collong)

        if dither_pattern_long == 'FULL3TIGHT' and mosaic == 'No':
            shiftv2 = [0.0, -58.0, 58.0]
            shiftv3 = [0.0, -7.5, 7.5]
            v2 = v2_0
            v3 = v3_0
            # determine center of rotation
            xa = (v2[0] + v2[2]) / 2.0
            ya = (v3[0] + v3[2]) / 2.0
            xb = (v2[5] + v2[7]) / 2.0
            yb = (v3[5] + v3[7]) / 2.0
            xr = ((xa + xb) / 2.)
            yr = ((ya + yb) / 2.)
            xr0 = xr
            yr0 = yr
            ra0 = ra_long
            dec0 = dec_long
            pa = theta_long+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            for shft in (0, 1, 2):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            create_footprint(
                inputfile,
                myv2,
                myv3,
                6,
                outdir+'/ds9-long-threetight.reg',
                collong)

        if dither_pattern_long == 'FULL6' and mosaic == 'No':
            shiftv2 = [-72.0, -43.0, -14.0, 15.0, 44.0, 73.0]
            shiftv3 = [-30.0, -18.0, -6.0, 6.0, 18.0, 30.0]
            # determine center of rotation
            v2 = v2_0 + 73.0
            v3 = v3_0 + 30.0
            xa = (v2[0] + v2[2]) / 2.0
            ya = (v3[0] + v3[2]) / 2.0
            v2 = v2_0 - 72.0
            v3 = v3_0 - 30.0
            xb = (v2[5] + v2[7]) / 2.0
            yb = (v3[5] + v3[7]) / 2.0
            xr = ((xa + xb) / 2.)
            yr = ((ya + yb) / 2.)

            v2 = v2_0
            v3 = v3_0

            xr0 = xr
            yr0 = yr
            ra0 = ra_long
            dec0 = dec_long
            pa = theta_long+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            for shft in (0, 1, 2, 3, 4, 5):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            create_footprint(
                inputfile,
                myv2,
                myv3,
                12,
                outdir+'/ds9-long-six.reg',
                collong)


        if (dither_pattern_long == 'FULL3' or dither_pattern_long ==
                'FULL3TIGHT' or dither_pattern_long == 'None') and mosaic == 'Yes':
            if dither_pattern_long == 'FULL3':
                shiftv3 = [0.0, -23.5, 23.5, usershiftv3 + 0.0,
                           usershiftv3 - 23.5, usershiftv3 + 23.5]
            if dither_pattern_long == 'FULL3TIGHT':
                shiftv3 = [0.0, -7.5, 7.5, usershiftv3 + 0.0,
                           usershiftv3 - 7.5, usershiftv3 + 7.5]

            shiftv2 = [0.0, -58.0, 58.0, usershiftv2 + 0.0,
                       usershiftv2 - 58.0, usershiftv2 + 58.0]
            if (dither_pattern_long == 'None'):
                shiftv3 = [0.0, usershiftv3 + 0.0]
                shiftv2 = [0.0, usershiftv2 + 0.0]

            v2 = v2_0
            v3 = v3_0
            # determine center of rotation
            v2 = np.append(v2, v2 + usershiftv2)
            v3 = np.append(v3, v3 + usershiftv3)
            ascii.write([v2, v3], 'values.dat')

            xa = (v2[0] + v2[2]) / 2.0
            ya = (v3[0] + v3[2]) / 2.0
            xb = (v2[5] + v2[7]) / 2.0
            yb = (v3[5] + v3[7]) / 2.0
            xr1 = ((xa + xb) / 2.)
            yr1 = ((ya + yb) / 2.)
            xa = (v2[10] + v2[12]) / 2.0
            ya = (v3[10] + v3[12]) / 2.0
            xb = (v2[15] + v2[17]) / 2.0
            yb = (v3[15] + v3[17]) / 2.0
            xr2 = ((xa + xb) / 2.)
            yr2 = ((ya + yb) / 2.)

            xr = ((xr1 + xr2) / 2.)
            yr = ((yr1 + yr2) / 2.)

            xr0 = xr
            yr0 = yr
            ra0 = ra_long
            dec0 = dec_long
            pa = theta_long+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            for shft in range(len(shiftv2)):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                #print(v20, v30)
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            if (dither_pattern_long == 'None'):
                napertures = 4
            if (dither_pattern_long ==
                    'FULL3' or dither_pattern_long == 'FULL3TIGHT'):
                napertures = 12
            create_footprint(
                inputfile,
                myv2,
                myv3,
                napertures,
                outdir+'/ds9-long-mosaic.reg',
                collong)


#-------------------------------------------------------------------
# nircam short
    if plot_short == 'Yes':
        print('processing NIRCAM SWC')
       
        v2sh, v3sh, aper, v2ref, v3ref = read_table(os.path.join(PKG_DATA_DIR, 'table-nircam-short.txt'))
        v2sh = np.array(v2sh, np.float_)
        v3sh = np.array(v3sh, np.float_)

#--------------------------------------------------------------------------  
#  here i need to transform ra dec to float          06JUN2017    LEONARDO
    
        print('using NIRCAm RA  :',ra_long)
        print('using NIRCam DEC :',dec_long)
        ra_short = ra_long
        dec_short = dec_long

        if (' ' in ra_long) and (' ' in dec_long) == True:   
           # it recognizes that the string has the format   hh mm ss.sss
           b = ra_long+' '+dec_long
           c = SkyCoord(b,unit=(u.hourangle, u.deg))
           #   transform to degrees to be used in the rest of the code
           ra = c.ra.deg 
           dec = c.dec.deg
           ra_short = np.array(ra, np.float_)
           dec_short = np.array(dec, np.float_)

        
        else:                          #   string is in units of degrees
           ra_short = np.array(ra_short, np.float_)
           dec_short = np.array(dec_short, np.float_)
#----------------------------------------------------------------------------          

        create_footprint_center(
            inputfile,
            ra_short,
            dec_short,
            outdir+'/ds9-short-centre.reg',
            colshort)

        if dither_pattern_short == 'None':
            x2 = v2sh
            x3 = v3sh
            v2 = v2sh
            v3 = v3sh
            # determine center of rotation
            # search point in the center of each aperture
            xsh1 = (x2[0] + x2[2]) / 2.0
            ysh1 = (x3[0] + x3[2]) / 2.0
            xsh2 = (x2[5] + x2[7]) / 2.0
            ysh2 = (x3[5] + x3[7]) / 2.0
            xsh3 = (x2[20] + x2[22]) / 2.0
            ysh3 = (x3[20] + x3[22]) / 2.0
            xsh4 = (x2[25] + x2[27]) / 2.0
            ysh4 = (x3[25] + x3[27]) / 2.0

            # calculate center for rotation
            xr13 = (xsh1 + xsh3) / 2.0
            yr13 = (ysh1 + ysh3) / 2.0
            xr24 = (xsh2 + xsh4) / 2.0
            yr24 = (ysh2 + ysh4) / 2.0
            xr = (xr13 + xr24) / 2.
            yr = (yr13 + yr24) / 2.

            xr0 = xr
            yr0 = yr
            ra0 = ra_short
            dec0 = dec_short
            pa = theta_short+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            v20 = xr0
            v30 = yr0
            m = attitude(v20, v30, ra0, dec0, pa)
            for k in range(len(v2)):
                a = pointing(m, v2[k], v3[k])
                myv2 = np.append(myv2, a[0])
                myv3 = np.append(myv3, a[1])
            myv2 = np.array(myv2)
            myv3 = np.array(myv3)
            #print(myv2)
            create_footprint(
                inputfile,
                myv2,
                myv3,
                8,
                outdir+'/ds9-short-no.reg',
                colshort)


        #                                                                          8NIRSPEC  15DEC2017
        if dither_pattern_short == '8NIRSPEC' :
            mosaic == 'No'
            shiftv2 = [-24.6, -24.4, 24.6, 24.4, 24.6, 24.4, -24.6, -24.4]
            shiftv3 = [-64.1, -89.0, -88.8, -63.9, 64.1, 89.0, 88.8, 63.9 ]
            x2 = v2sh
            x3 = v3sh
            v2 = v2sh
            v3 = v3sh
            # determine center of rotation
            # search point in the center of each aperture
            xsh1 = (x2[0] + x2[2]) / 2.0
            ysh1 = (x3[0] + x3[2]) / 2.0
            xsh2 = (x2[5] + x2[7]) / 2.0
            ysh2 = (x3[5] + x3[7]) / 2.0
            xsh3 = (x2[20] + x2[22]) / 2.0
            ysh3 = (x3[20] + x3[22]) / 2.0
            xsh4 = (x2[25] + x2[27]) / 2.0
            ysh4 = (x3[25] + x3[27]) / 2.0
            # calculate center for rotation
            xr13 = (xsh1 + xsh3) / 2.0
            yr13 = (ysh1 + ysh3) / 2.0
            xr24 = (xsh2 + xsh4) / 2.0
            yr24 = (ysh2 + ysh4) / 2.0
            xr = (xr13 + xr24) / 2.
            yr = (yr13 + yr24) / 2.
            xr0 = xr
            yr0 = yr
            ra0 = ra_short
            dec0 = dec_short
            pa = theta_short+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            for shft in range(len(shiftv2)):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            #print(len(myv2))    
            create_footprint(
                inputfile,
                myv2,
                myv3,
                64,
                outdir+'/ds9-short-8nirspec.reg',
                colshort)



        if dither_pattern_short == 'FULL3':
            shiftv2 = [0.0, -58.0, 58.0]
            shiftv3 = [0.0, -23.5, 23.5]
            x2 = v2sh
            x3 = v3sh
            v2 = v2sh
            v3 = v3sh
            # determine center of rotation
            # search point in the center of each aperture
            xsh1 = (x2[0] + x2[2]) / 2.0
            ysh1 = (x3[0] + x3[2]) / 2.0
            xsh2 = (x2[5] + x2[7]) / 2.0
            ysh2 = (x3[5] + x3[7]) / 2.0
            xsh3 = (x2[20] + x2[22]) / 2.0
            ysh3 = (x3[20] + x3[22]) / 2.0
            xsh4 = (x2[25] + x2[27]) / 2.0
            ysh4 = (x3[25] + x3[27]) / 2.0
            # calculate center for rotation
            xr13 = (xsh1 + xsh3) / 2.0
            yr13 = (ysh1 + ysh3) / 2.0
            xr24 = (xsh2 + xsh4) / 2.0
            yr24 = (ysh2 + ysh4) / 2.0
            xr = (xr13 + xr24) / 2.
            yr = (yr13 + yr24) / 2.
            xr0 = xr
            yr0 = yr
            ra0 = ra_short
            dec0 = dec_short
            pa = theta_short+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            for shft in (0, 1, 2):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            create_footprint(
                inputfile,
                myv2,
                myv3,
                24,
                outdir+'/ds9-short-three.reg',
                colshort)

        if dither_pattern_short == 'FULL3TIGHT':
            shiftv2 = [0.0, -58.0, 58.0]
            shiftv3 = [0.0, -7.5, 7.5]
            x2 = v2sh
            x3 = v3sh
            v2 = v2sh
            v3 = v3sh
            # determine center of rotation
            # search point in the center of each aperture
            xsh1 = (x2[0] + x2[2]) / 2.0
            ysh1 = (x3[0] + x3[2]) / 2.0
            xsh2 = (x2[5] + x2[7]) / 2.0
            ysh2 = (x3[5] + x3[7]) / 2.0
            xsh3 = (x2[20] + x2[22]) / 2.0
            ysh3 = (x3[20] + x3[22]) / 2.0
            xsh4 = (x2[25] + x2[27]) / 2.0
            ysh4 = (x3[25] + x3[27]) / 2.0

            # calculate center for rotation
            xr13 = (xsh1 + xsh3) / 2.0
            yr13 = (ysh1 + ysh3) / 2.0
            xr24 = (xsh2 + xsh4) / 2.0
            yr24 = (ysh2 + ysh4) / 2.0
            xr = (xr13 + xr24) / 2.
            yr = (yr13 + yr24) / 2.
            xr0 = xr
            yr0 = yr
            ra0 = ra_short
            dec0 = dec_short
            pa = theta_short+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            for shft in (0, 1, 2):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            create_footprint(
                inputfile,
                myv2,
                myv3,
                24,
                outdir+'/ds9-short-threetight.reg',
                colshort)

        if dither_pattern_short == 'FULL6':
            shiftv2 = [-72.0, -43.0, -14.0, 15.0, 44.0, 73.0]
            shiftv3 = [-30.0, -18.0, -6.0, 6.0, 18.0, 30.0]

            # determine center of rotation using info from long dither pattern
            v2sh, v3sh, aper, v2ref, v3ref = read_table(os.path.join(PKG_DATA_DIR, 'table-nircam-long.txt'))
            v2sh = np.array(v2sh, np.float_)
            v3sh = np.array(v3sh, np.float_)

            v2 = v2sh + 73.0
            v3 = v3sh + 30.0
            xa = (v2[0] + v2[2]) / 2.0
            ya = (v3[0] + v3[2]) / 2.0
            v2 = v2sh - 72.0
            v3 = v3sh - 30.0
            xb = (v2[5] + v2[7]) / 2.0
            yb = (v3[5] + v3[7]) / 2.0
            xr = ((xa + xb) / 2.)
            yr = ((ya + yb) / 2.)

            v2sh, v3sh, aper, v2ref, v3ref = read_table(os.path.join(PKG_DATA_DIR, 'table-nircam-short.txt'))
            v2sh = np.array(v2sh, np.float_)
            v3sh = np.array(v3sh, np.float_)

            v2 = v2sh
            v3 = v3sh

            xr0 = xr
            yr0 = yr
            ra0 = ra_short
            dec0 = dec_short
            pa = theta_short+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            for shft in (0, 1, 2, 3, 4, 5):
                v20 = xr0 + (shiftv2[shft])  # here we shift
                v30 = yr0 + (shiftv3[shft])  # here we shift
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            create_footprint(
                inputfile,
                myv2,
                myv3,
                48,
                outdir+'/ds9-short-six.reg',
                colshort)


#  mosaic  short wavelength channel

        if (dither_pattern_short == 'FULL3' or dither_pattern_short ==
                'FULL3TIGHT' or dither_pattern_short == 'None') and mosaic == 'Yes':
            if dither_pattern_short == 'FULL3':
                shiftv3 = [0.0, -23.5, 23.5, usershiftv3 + 0.0,
                           usershiftv3 - 23.5, usershiftv3 + 23.5]
            if dither_pattern_short == 'FULL3TIGHT':
                shiftv3 = [0.0, -7.5, 7.5, usershiftv3 + 0.0,
                           usershiftv3 - 7.5, usershiftv3 + 7.5]

            shiftv2 = [0.0, -58.0, 58.0, usershiftv2 + 0.0,
                       usershiftv2 - 58.0, usershiftv2 + 58.0]

            if (dither_pattern_short == 'None'):
                shiftv3 = [0.0, usershiftv3 + 0.0]
                shiftv2 = [0.0, usershiftv2 + 0.0]
            v2 = v2sh
            v3 = v3sh

            # determine center of rotation
            v2 = np.append(v2, v2 + usershiftv2)
            v3 = np.append(v3, v3 + usershiftv3)
            ascii.write([v2, v3], 'values.dat')

            xa = (v2[0] + v2[22]) / 2.0
            ya = (v3[0] + v3[22]) / 2.0
            xb = (v2[8] + v2[26]) / 2.0
            yb = (v3[8] + v3[26]) / 2.0
            xr1 = ((xa + xb) / 2.)
            yr1 = ((ya + yb) / 2.)
            xa = (v2[40] + v2[62]) / 2.0
            ya = (v3[40] + v3[62]) / 2.0
            xb = (v2[48] + v2[66]) / 2.0
            yb = (v3[48] + v3[66]) / 2.0
            xr2 = ((xa + xb) / 2.)
            yr2 = ((ya + yb) / 2.)

            xr = ((xr1 + xr2) / 2.)
            yr = ((yr1 + yr2) / 2.)

            xr0 = xr
            yr0 = yr
            ra0 = ra_short
            dec0 = dec_short
            pa = theta_short+ 0.0265    #  12MAR2018  NRCALL_FULL_V2IdlYang = -0.0265
            myv2 = []
            myv3 = []
            v2 = v2sh
            v3 = v3sh
            for shft in range(len(shiftv2)):
                v20 = xr0 - (shiftv2[shft])  # here we shift
                v30 = yr0 - (shiftv3[shft])  # here we shift
                #print(v20, v30)
                m = attitude(v20, v30, ra0, dec0, pa)
                for k in range(len(v2)):
                    a = pointing(m, v2[k], v3[k])
                    myv2 = np.append(myv2, a[0])
                    myv3 = np.append(myv3, a[1])
                myv2 = np.array(myv2)
                myv3 = np.array(myv3)
            if (dither_pattern_short == 'None'):
                napertures = 16
            if (dither_pattern_short ==
                    'FULL3' or dither_pattern_short == 'FULL3TIGHT'):
                napertures = 48
            create_footprint(
                inputfile,
                myv2,
                myv3,
                napertures,
                outdir+'/ds9-short-mosaic.reg',
                colshort)
#  mosaic  short wavelength channel

    # Start xpans prior to running DS9
    pyds9.ds9_xpans()

    # Run DS9
    d = pyds9.DS9()
    d.set('tile yes')
    d.set('frame 1')
    d.set('cmap ' + ds9cmap)
    d.set('scale limits ' + ds9limmin + ' ' + ds9limmax)
    d.set('scale ' + ds9scale)
    d.set('file ' + inputfile)
    # load regions
    if plot_long == 'Yes':
        d.set('regions ' +outdir+'/ds9-long-centre.reg')
        if mosaic == 'No':
            if dither_pattern_long == 'FULL3TIGHT':
                d.set('regions ' +outdir+'/ds9-long-threetight.reg')
            if dither_pattern_long == 'FULL3':
                d.set('regions ' +outdir+'/ds9-long-three.reg')
            if dither_pattern_long == '8NIRSPEC':
                d.set('regions ' +outdir+'/ds9-long-8nirspec.reg')
            if dither_pattern_long == 'FULL6':
                d.set('regions ' +outdir+'/ds9-long-six.reg')
            if dither_pattern_long == 'None':
                d.set('regions ' +outdir+'/ds9-long-no.reg')
        if mosaic == 'Yes' and dither_pattern_long != '8NIRSPEC' :
            d.set('regions ' +outdir+'/ds9-long-mosaic.reg')
        if mosaic == 'Yes' and dither_pattern_long == '8NIRSPEC' :
            print('Mosaic pattern is disabled for 8NIRSPEC dither pattern')
    if plot_short == 'Yes':
        d.set('regions ' +outdir+'/ds9-short-centre.reg')
        if mosaic == 'No':
            if dither_pattern_short == 'FULL3TIGHT':
                d.set('regions ' +outdir+'/ds9-short-threetight.reg')
            if dither_pattern_short == 'FULL3':
                d.set('regions ' +outdir+'/ds9-short-three.reg')
            if dither_pattern_short == '8NIRSPEC':
                d.set('regions ' +outdir+'/ds9-short-8nirspec.reg')
            if dither_pattern_short == 'FULL6':
                d.set('regions ' +outdir+'/ds9-short-six.reg')
            if dither_pattern_short == 'None':
                d.set('regions ' +outdir+'/ds9-short-no.reg')
        if mosaic == 'Yes' and dither_pattern_short != '8NIRSPEC':
            d.set('regions ' +outdir+'/ds9-short-mosaic.reg')
        if mosaic == 'Yes' and dither_pattern_short == '8NIRSPEC':
            print('Mosaic pattern is disabled for 8NIRSPEC dither pattern')

    if plot_msa == 'Yes':
        d.set('regions ' +outdir+'/ds9-msa.reg')
        d.set('regions ' +outdir+'/ds9-msa-centre.reg')

    if plot_sources == 'Yes':
        if flagsources == 3:
            #print(plot_sources)
            d.set('regions ' +outdir+'/ds9-sources-fillers.reg')
            d.set('regions ' +outdir+'/ds9-sources-primary.reg')
        if flagsources == 2:
            #print(plot_sources)
            d.set('regions ' +outdir+'/ds9-sources.reg')



#! /usr/bin/env python
# Version 1. August 2, 2010
# Version 2. August 3, 2010
#   Got rid of degrees trig functions
from __future__ import absolute_import, division, print_function
from math import atan2, acos, cos, sin, pi

D2R = pi / 180.
R2D = 180. / pi
PI2 = 2. * pi
epsilon = 23.43929 * D2R  # obliquity of the ecliptic J2000


def unit_limit(x): return min(max(-1., x), 1.)


def pa(tgt_c1, tgt_c2, obj_c1, obj_c2):
    """calculates position angle of object at tgt position."""
    y = cos(obj_c2) * sin(obj_c1 - tgt_c1)
    x = (sin(obj_c2) * cos(tgt_c2) - cos(obj_c2) *
         sin(tgt_c2) * cos(obj_c1 - tgt_c1))
    p = atan2(y, x)
    if p < 0.:
        p += PI2
    if p >= PI2:
        p -= PI2
    return p


def delta_pa_no_roll(pos1_c1, pos1_c2, pos2_c1, pos2_c2):
    """Calculates the change in position angle between two positions with no roll about V1"""
    u = (sin(pos1_c2) + sin(pos2_c2)) * sin(pos2_c1 - pos1_c1)
    v = cos(pos2_c1 - pos1_c1) + cos(pos1_c2) * cos(pos2_c2) + \
        sin(pos1_c2) * sin(pos2_c2) * cos(pos2_c1 - pos1_c1)
    return atan2(u, v)


def dist(obj1_c1, obj1_c2, obj2_c1, obj2_c2):
    """angular distance betrween two objects, positions specified in spherical coordinates."""
    x = cos(obj2_c2) * cos(obj1_c2) * cos(obj2_c1 -
                                          obj1_c1) + sin(obj2_c2) * sin(obj1_c2)
    return acos(unit_limit(x))



#! /usr/bin/env python
from __future__ import absolute_import, division, print_function

import os
import sys
import math

from . import ephemeris_old2x as EPH
from . import PKG_DATA_DIR

D2R = math.pi / 180.  # degrees to radians
R2D = 180. / math.pi  # radians to degrees
PI2 = 2. * math.pi   # 2 pi


def unit_limit(x): return min(max(-1., x), 1.)  # forces value to be in [-1,1]


def convert_ddmmss_to_float(astring):
    aline = aline.split(astring, ':')
    d = float(aline[0])
    m = float(aline[1])
    s = float(aline[2])
    hour_or_deg = (s / 60. + m) / 60. + d
    return hour_or_deg


def bound_angle(ang):
    while ang < 0.:
        ang += 360.
    while ang > 360.:
        ang -= 360.
    return ang


def angular_sep(obj1_c1, obj1_c2, obj2_c1, obj2_c2):
    """angular distance betrween two objects, positions specified in spherical coordinates."""
    x = math.cos(obj2_c2) * math.cos(obj1_c2) * math.cos(obj2_c1 -
                                                         obj1_c1) + math.sin(obj2_c2) * math.sin(obj1_c2)
    return math.acos(unit_limit(x))


def calc_ecliptic_lat(ra, dec):
    NEP_ra = 270.000000 * D2R
    NEP_dec = 66.560708 * D2R
    a_sep = angular_sep(ra, dec, NEP_ra, NEP_dec)
    ecl_lat = math.pi / 2. - a_sep
    return ecl_lat


def sun_pitch(aV):
    return math.atan2(aV.x, -aV.z)


def sun_roll(aV):
    return math.asin(-aV.y)


def allowed_max_sun_roll(sun_p):
    abs_max_sun_roll = 5.2 * D2R
    if sun_p > 2.5 * D2R:
        max_sun_roll = abs_max_sun_roll - 1.7 * D2R * \
            (sun_p - 2.5 * D2R) / (5.2 - 2.5) / D2R
    else:
        max_sun_roll = abs_max_sun_roll
    max_sun_roll -= 0.1 * D2R  # Pad away from the edge
    return max_sun_roll


def allowed_max_vehicle_roll(sun_ra, sun_dec, ra, dec):
    vehicle_pitch = math.pi / 2. - angular_sep(sun_ra, sun_dec, ra, dec)
    sun_roll = 5.2 * D2R
    last_sun_roll = 0.
    while abs(sun_roll - last_sun_roll) > 0.0001 * D2R:
        last_sun_roll = sun_roll
        sun_pitch = math.asin(
            unit_limit(
                math.sin(vehicle_pitch) /
                math.cos(last_sun_roll)))
        sun_roll = allowed_max_sun_roll(sun_pitch)
        # print sun_roll*R2D,sun_pitch*R2D,vehicle_pitch*R2D
    max_vehicle_roll = math.asin(
        unit_limit(
            math.sin(sun_roll) /
            math.cos(vehicle_pitch)))
    return max_vehicle_roll


def rollangle(ra, dec, outdir):
    NRCALL_FULL_V2IdlYang = -0.0265
    NRS_FULL_MSA_V3IdlYang = 137.4874
    NIS_V3IdlYang = -0.57
    MIRIM_FULL_V3IdlYang = 5.0152
    FGS1_FULL_V3IdlYang = -1.2508

    ECL_FLAG = False

    A_eph = EPH.Ephemeris(os.path.join(
        PKG_DATA_DIR, "horizons_EM_L2_wrt_Sun_2018_2022.txt"), ECL_FLAG)

    # search_start = 58484.00000000  #Jan 1, 2019
    search_start = 58392.00000000  # Oct 1, 2018         LEONARDO

    if search_start < A_eph.amin:
        print("Warning, search start time is earlier than ephemeris start.")
        search_start = A_eph.amin + 1

    scale = 10
    span = 365 * 3

    pa = "X"
    ra = float(ra) * D2R
    dec = float(dec) * D2R

    if len(sys.argv) > 3:
        pa = float(sys.argv[3]) * D2R
    # print "Checked interval [%f,%f] MJD" % (search_start,search_start+span)
    if pa == "X":
        iflag_old = A_eph.in_FOR(search_start, ra, dec)
    # print "|           Window [days]              |         Normal V3 PA
    # [deg]|"
    else:
        iflag_old = A_eph.is_valid(search_start, ra, dec, pa)
    # print "|           Window [days]              |         Specified V3 PA
    # [deg]|"

    # print "   Start[MJD]      End[MJD]    Duration         Start         End
    # coord1        coord2"

    if iflag_old:
        twstart = search_start
    else:
        twstart = -1.
    iflip = False

    # Step througth the interval and find where target goes in/out of field of
    # regard.
    for i in range(1, span * scale + 1):
        adate = search_start + float(i) / float(scale)
        # iflag = A_eph.in_FOR(adate,ra,dec)
        if pa == "X":
            iflag = A_eph.in_FOR(adate, ra, dec)
        else:
            iflag = A_eph.is_valid(adate, ra, dec, pa)
        if iflag != iflag_old:
            iflip = True
            if iflag:
                if pa == "X":
                    twstart = A_eph.bisect_by_FOR(adate, adate - 0.1, ra, dec)
                else:
                    twstart = A_eph.bisect_by_attitude(
                        adate, adate - 0.1, ra, dec, pa)
            else:
                if pa == "X":
                    wend = A_eph.bisect_by_FOR(adate - 0.1, adate, ra, dec)
                else:
                    wend = A_eph.bisect_by_attitude(
                        adate - 0.1, adate, ra, dec, pa)
                if twstart > 0.:
                    wstart = twstart  # Only set wstart if wend is valid
                    if pa == "X":
                        pa_start = A_eph.normal_pa(wstart, ra, dec)
                        pa_end = A_eph.normal_pa(wend, ra, dec)
                    else:
                        pa_start = pa
                        pa_end = pa
                    # print "%13.5f %13.5f %11.2f %13.5f %13.5f %13.5f %13.5f "
                    # %
                    # (wstart,wend,wend-wstart,pa_start*R2D,pa_end*R2D,ra*R2D,dec*R2D)
            iflag_old = iflag

    if iflip and iflag:
        if pa == "X":
            pa_start = A_eph.normal_pa(twstart, ra, dec)
            pa_end = A_eph.normal_pa(adate, ra, dec)
        else:
            pa_start = pa
            pa_end = pa
    # print "%13.5f %13.5f %11.2f %13.5f %13.5f %13.5f %13.5f " %
    # (twstart,adate,adate-twstart,pa_start*R2D,pa_end*R2D,ra*R2D,dec*R2D)

    '''
        if iflip == False and iflag == True and pa == "X":
            if dec >0.:
                print "%13s %13s %11s %13.5f %13.5f %13.5f %13.5f " % ('CVZ','CVZ','CVZ',360.,0.,ra*R2D,dec*R2D)
            else:
                print "%13s %13s %11s %13.5f %13.5f %13.5f %13.5f " % ('CVZ','CVZ','CVZ',0.,360.,ra*R2D,dec*R2D)
    '''
    if 1 == 1:
        wstart = search_start
        wend = wstart + span
        istart = int(wstart)
        iend = int(wend)
        iflag = A_eph.in_FOR(wstart, ra, dec)
        tgt_is_in = False
        if iflag:
            tgt_is_in = True
        # print()
        # print()
        # print "             V3PA          NIRCam           NIRSpec         NIRISS           MIRI          FGS"
        # print "   MJD    min    max      min    max      min    max      min    max      min    max      min    max"
        # 58849.0 264.83 275.18 264.80 264.80  42.32  42.32 264.26 264.26
        # 269.84 269.84 263.58 263.58
        # verify that outdir exists

        if not os.path.exists(outdir):
            os.makedirs(outdir, mode=0o0755)
            print("creating directory {}".format(outdir))

        outputfile = os.path.join(outdir, 'v3pa_nircam_nirspec.txt')
        print(outputfile)

        with open(outputfile, "w") as fp:
            # pos = 0
            # for i in range(0,napertures):
            #        newline = 'polygon '+ str(x[pos])+ '  '+ str(y[pos])+ '  '+ str(x[pos+1])+ '  '+ str(y[pos+1])+ '  '+ str(x[pos+2])+ '  '+ str(y[pos+2])+ '  '+ str(x[pos+3])+ '  '+ str(y[pos+3])+ '  '+ str(x[pos+4])+ '  '+ str(y[pos+4])+ '  ' +  '# text={}'+'\n'
            #        fp.write(newline)
            #        pos = pos + 5

            for itime in range(istart, iend):
                atime = float(itime)
                iflag = A_eph.in_FOR(atime, ra, dec)
                # print atime,A_eph.in_FOR(atime,ra,dec)
                if iflag:
                    if not tgt_is_in:
                        print()
                    tgt_is_in = True

                    V3PA = A_eph.normal_pa(atime, ra, dec) * R2D
                    (sun_ra, sun_dec) = A_eph.sun_pos(atime)
                    max_boresight_roll = allowed_max_vehicle_roll(
                        sun_ra, sun_dec, ra, dec) * R2D

                    minV3PA = bound_angle(V3PA - max_boresight_roll)
                    maxV3PA = bound_angle(V3PA + max_boresight_roll)
                    # minNIRCam_PA = bound_angle(V3PA - max_boresight_roll + NRCALL_FULL_V2IdlYang)
                    # maxNIRCam_PA = bound_angle(V3PA - max_boresight_roll + NRCALL_FULL_V2IdlYang)
                    # minNIRSpec_PA = bound_angle(V3PA - max_boresight_roll + NRS_FULL_MSA_V3IdlYang)
                    # maxNIRSpec_PA = bound_angle(V3PA - max_boresight_roll + NRS_FULL_MSA_V3IdlYang)
                    # minNIRISS_PA = bound_angle(V3PA - max_boresight_roll + NIS_V3IdlYang)
                    # maxNIRISS_PA = bound_angle(V3PA - max_boresight_roll + NIS_V3IdlYang)
                    # minMIRI_PA = bound_angle(V3PA - max_boresight_roll + MIRIM_FULL_V3IdlYang)
                    # maxMIRI_PA = bound_angle(V3PA - max_boresight_roll + MIRIM_FULL_V3IdlYang)
                    # minFGS_PA = bound_angle(V3PA - max_boresight_roll + FGS1_FULL_V3IdlYang)
                    # maxFGS_PA = bound_angle(V3PA - max_boresight_roll + FGS1_FULL_V3IdlYang)

                    minNIRCam_PA = bound_angle(
                        V3PA - max_boresight_roll + NRCALL_FULL_V2IdlYang)
                    maxNIRCam_PA = bound_angle(
                        V3PA + max_boresight_roll + NRCALL_FULL_V2IdlYang)
                    minNIRSpec_PA = bound_angle(
                        V3PA - max_boresight_roll + NRS_FULL_MSA_V3IdlYang)
                    maxNIRSpec_PA = bound_angle(
                        V3PA + max_boresight_roll + NRS_FULL_MSA_V3IdlYang)
                    minNIRISS_PA = bound_angle(
                        V3PA - max_boresight_roll + NIS_V3IdlYang)
                    maxNIRISS_PA = bound_angle(
                        V3PA + max_boresight_roll + NIS_V3IdlYang)
                    minMIRI_PA = bound_angle(
                        V3PA - max_boresight_roll + MIRIM_FULL_V3IdlYang)
                    maxMIRI_PA = bound_angle(
                        V3PA + max_boresight_roll + MIRIM_FULL_V3IdlYang)
                    minFGS_PA = bound_angle(
                        V3PA - max_boresight_roll + FGS1_FULL_V3IdlYang)
                    maxFGS_PA = bound_angle(
                        V3PA + max_boresight_roll + FGS1_FULL_V3IdlYang)

                    # fmt = '%7.1f' + '   %6.2f %6.2f' * 6
                    # print(
                    #    fmt %
                    #    (atime,
                    #     minV3PA,
                    #     maxV3PA,
                    #     minNIRCam_PA,
                    #     maxNIRCam_PA,
                    #     minNIRSpec_PA,
                    #     maxNIRSpec_PA,
                    #     minNIRISS_PA,
                    #     maxNIRISS_PA,
                    #     minMIRI_PA,
                    #     maxMIRI_PA,
                    #     minFGS_PA,
                    #     maxFGS_PA))
                    newline = str(atime) + '   ' + str(minV3PA) + \
                        ' ' + str(maxV3PA) + '  ' + str(minNIRCam_PA) + \
                        '  ' + str(maxNIRCam_PA) + '  ' + str(minNIRSpec_PA) + \
                        '  ' + str(maxNIRSpec_PA) + '  \n'

                    fp.write(newline)

                else:
                    tgt_is_in = False

    return ra, dec



#! /usr/bin/env python
from __future__ import absolute_import, division, print_function

import datetime
import matplotlib
matplotlib.use("TkAgg")
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
import numpy as np
from astropy import units as u
from astropy.coordinates import SkyCoord

from astropy.io import ascii
from astropy.time import Time
from .find_tgt_info import *


def plottimeline(ra_msa='202.47',
                 dec_msa='47.2',
                 theta_msa=0.0,
                 outdir='/Users/myname/Desktop/'):

  
  #--------------------------------------------------------------------------  
  #  here i need to transform ra dec to float      06JUN2017    LEONARDO
    
    print('using NIRSpec RA  :',ra_msa)
    print('using NIRSpec DEC :',dec_msa)

    if (' ' in ra_msa) and (' ' in dec_msa) == True:   
       # it recognizes that the string has the format   hh mm ss.sss
       b = ra_msa+' '+dec_msa
       c = SkyCoord(b,unit=(u.hourangle, u.deg))
       #   transform to degrees to be used in the rest of the code
       ra = c.ra.deg 
       dec = c.dec.deg
       ra_msa = np.array(ra, np.float_)
       dec_msa = np.array(dec, np.float_)

    
    else:                          #   string is in units of degrees
       ra_msa = np.array(ra_msa, np.float_)
       dec_msa = np.array(dec_msa, np.float_)
#----------------------------------------------------------------------------  


    # here we use Wayne's code to calculate the rollangle for NIRCam and
    # NIRSpec
    rollangle(float(ra_msa), float(dec_msa),outdir)
    # output goes to screen and to a text file with the name v3pa_nircam_nirspec.txt

    # read v3pa_nircam_nirspec.txt
    data = ascii.read(outdir+'/v3pa_nircam_nirspec.txt')
    # time expressed in JDTDB Julian Day Number, Barycentric Dynamical Time
    x = np.array(data['col1']) + 2400000.5
    t = Time(x, format='jd')
    ti = t.datetime
    timeplot = np.array(t.datetime)

    # we read the values to plot
    minv2v3 = np.array(data['col2'])  #minimum possible value for v2v3
    maxv2v3 = np.array(data['col3'])  #maximum possible value for v2v3 
    minnircam = np.array(data['col4'])  # minimum possible value for nircam
    maxnircam = np.array(data['col5'])  # maximum possible value for nircam
    minnirspec = np.array(data['col6'])  # minimum possible value for nirspec
    maxnirspec = np.array(data['col7'])  # maximum possible value for nirspec

    fig = plt.figure(
        figsize=(
            10,
            5),
        dpi=120,
        edgecolor=None,
        facecolor='white')
    plt.grid(True)

    launch = datetime.date(2019, 3, 1)

    numdays = 90 + 365 * 4

    plt.plot_date(timeplot,minv2v3,color='yellow', marker='o',markersize=5)
    plt.plot_date(timeplot,maxv2v3,color='yellow', marker='o',markersize=5)

    plt.plot_date(timeplot,minnircam,color='blue', marker='s',markersize=1)
    plt.plot_date(timeplot,maxnircam,color='blue', marker='s',markersize=1)

    plt.plot_date(timeplot,maxnirspec,color='firebrick',  marker='.',markersize=2)
    plt.plot_date(timeplot,minnirspec,color='firebrick' , marker='.',markersize=2)

    red_patch = mpatches.Patch(color='blue', label='NIRCam')
    gre_patch = mpatches.Patch(color='firebrick', label='NIRSPec')
    bla_patch = mpatches.Patch(color='yellow', label='V3PA')


    plt.legend(handles=[red_patch,gre_patch,bla_patch])
    plt.gcf().autofmt_xdate()
   # plt.title('Aperture Position Angle')

    commissioning_start = datetime.date(2019, 3, 1)
    commissioning_end = datetime.date(2019, 8, 31)
    '''
    plt.axvspan(
        commissioning_start,
        commissioning_end,
        ymin=0.0,
        ymax=360,
        alpha=0.99,
        color='steelblue')
    #plt.text(commissioning_start, 378, 'JWST')
    plt.text(commissioning_start, 365, 'commissioning')

    cycle1_start = datetime.date(2019, 9, 1)
    cycle1_end = datetime.date(2020, 8, 31)
    plt.axvspan(
        cycle1_start,
        cycle1_end,
        ymin=0.0,
        ymax=360,
        alpha=0.2,
        color='grey')
    plt.text(datetime.date(2019, 9, 1), 370, 'science cycle 1')

    cycle2_start = datetime.date(2020, 9, 1)
    cycle2_end = datetime.date(2021, 8, 31)
    plt.axvspan(
        cycle2_start,
        cycle2_end,
        ymin=0.0,
        ymax=360,
        alpha=1.0,
        color='white')
    plt.text(datetime.date(2020, 9, 1), 370, 'science cycle 2')
    '''
    cycle3_start = datetime.date(2021, 9, 1)
    cycle3_end = datetime.date(2022, 8, 31)
    #plt.axvspan(
    #    cycle3_start,
    #    cycle3_end,
    #    ymin=0.0,
    #    ymax=360,
    #    alpha=0.2,
    #    color='grey')
    #plt.text(datetime.date(2021, 8, 1), 370, 'cycle 3 science')

    plt.text(commissioning_start, 399, 'RA  : ' + str(ra_msa))
    plt.text(commissioning_start, 385, 'DEC : ' + str(dec_msa))

    plt.xlim([commissioning_start, cycle3_start])
    plt.ylim([0, 360])
    plt.ylabel('Aperture PA')
    plt.show()



default_config = dict(
    #
    # Input filenames
    #
    fits_name='/Users/change-me/',
    cat_name='',
    out_dir='/Users/change-me/',
    #
    # to plot or not to plot ?
    #
    plot_names=['plot'],
    plot_msa="No",
    plot_short="No",
    plot_long="No",
    plot_src="No",
    dither="None",
    mosaic="No",
    #
    # color lines
    #
    color_msa='Red',
    color_short='Green',
    color_long='Blue',
    #
    # fiducial point equatorial coordinates
    # in degrees
    #
    ra_nircam=202.46959,
    dec_nircam=47.195187,
    theta_nircam=0.0,

    ra_nirspec=202.46959,
    dec_nirspec=47.195187,
    theta_nirspec=0.0,
    #
    # nircam dither offset
    #
    off_h=10.0,
    off_v=0.0,
    #
    # DS9 display
    #
    cmap='grey',    # grey, red, green, blue, heat
    lim_min=0.0,
    lim_max=100.0,
    scale='log'     # linear, log, power, squared
)



from os.path import abspath, dirname, expanduser, join


PKG_DIR = dirname(abspath(__file__))
PKG_DATA_DIR = join(PKG_DIR, 'data')
CONFIG_DIR = join(expanduser('~'), '.jwst_footprints')
CONFIG_FILE = join(CONFIG_DIR, 'config.json')



#! /usr/bin/env python
"""Module for general rotations library."""
from __future__ import absolute_import, division, print_function

import math
#from . import quaternionx
from . import math_extensionsx as math2


class GalacticPole (object):
    """Represents coordinates of galactic pole."""

    def __init__(self, latitude, longitude, ascending_node):
        """Initializes the coordinates of the galactic pole.

        latitude = latitude of pole, in degrees
        longitude = longitude of pole, in degrees
        ascending_node = ascending node of pole, in degrees."""

        # Arguments specified in degrees, but values represented in radians.
        self.latitude = math.radians(latitude)
        self.longitude = math.radians(longitude)
        self.anode = math.radians(ascending_node)

    def __str__(self):
        """Returns string representation of the galactic pole."""

        # Convert attributes back into degrees for readability.
        return('GalacticPole: latitude: %.3fD, longitude: %.3fD, anode: %.3fD'
               % (degrees(self.latitude), degrees(self.longitude), degrees(self.anode)))


# supports transformation to galactic coordinates
NGP = GalacticPole(192.859508, 27.128336, 32.932)


class NumericList (list):
    """List class that supports multiplication.  Only valid for numbers."""

    def __mul__(L1, L2):
        """Take the dot product of two numeric lists.
        Not using Vector for this because it is limited to three dimensions.
        Lists must have the same number of elements."""

        return(sum(map(lambda x, y: x * y, L1, L2)))


class Matrix (list):
    """Class to encapsulate matrix data and methods.

        A matrix is simply a list of lists that correspond to rows of the matrix.
        This is just intended to handle simple multiplication and vector rotations.
        For anything more advanced or computationally intensive, Python library routines
        should be used."""

    def __init__(self, rows):
        """Constructor for a matrix.

        This accepts a list of rows.
        It is assumed the rows are all of the same length."""

        for row in rows:
            self.append(NumericList(row))  # copy list

    def __str__(self):
        """Returns a string representation of the matrix."""

        return_str = 'Matrix:'

        for row_index in range(len(self)):
            row_str = 'Row %d: ' % (row_index + 1)
            row = self[row_index]

            for col_index in range(len(row)):
                row_str = row_str + '%6.3f  ' % (row[col_index])

            return_str = return_str + '\n' + row_str

        return(return_str)

    def element(self, row_index, col_index):
        """Returns an element of the matrix indexed by row and column.

        Indices begin with 0."""

        return ((self[row_index])[col_index])

    def row(self, row_index):
        """Returns a specified row of the matrix."""

        return(self[row_index])

    def column(self, col_index):
        """Returns a specified column of the matrix as a numeric list."""

        return(NumericList([row[col_index] for row in self]))

    def num_rows(self):
        """Returns the number of rows in the matrix."""

        return(len(self))

    def num_cols(self):
        """Returns the number of columns in the matrix."""

        return (len(self[0]))  # assumes all rows of equal length

    def get_cols(self):
        """Returns list of all columns in a matrix."""

        return ([self.column(col_index)
                 for col_index in range(0, self.num_cols())])

    def __mul__(m1, m2):
        """Multiplies two Matrix objects and returns the resulting matrix.

        Number of rows in m1 must equal the number of columns in m2."""

        result_rows = []

        # Iterate over the rows in m1.  The first column of row i is formed by
        # multiplying the ith row of m1 by the first column of m2.  The second
        # column is formed by muliplying the ith row of m1 by the second column
        # of m2, etc.
        for row in m1:
            new_row = []

            for col in m2.get_cols():
                new_row.append(row * col)

            result_rows.append(new_row)

        return (Matrix(result_rows))


class Vector (object):
    "Class to encapsulate vector data and operations."

    def __init__(self, x=0.0, y=0.0, z=0.0):
        """Constructor for a three-dimensional vector.

            Note that two-dimensional vectors can be constructed by omitting one of
            the coordinates, which will default to 0."""

        self.x = x  # Cartesian x coordinate
        self.y = y  # Cartesian y coordinate
        self.z = z  # Cartesian z coordinate

    def __str__(self):  # Called when used in print statement
        """Returns a string representation of the vector."""
        return('Vector: x: %.3f, y: %.3f, z: %.3f'
               % (self.x, self.y, self.z))

    def set_eq(self, x=None, y=None, z=None):
        """Assigns new value to vector.

        Arguments are now optional to permit this to be used with 2D vectors
        or to modify any subset of coordinates."""

        if (x is not None):
            self.x = x
        if (y is not None):
            self.y = y
        if (z is not None):
            self.z = z

    def length(self):
        """Returns magnitude of the vector """
        return(math.sqrt(self.x * self.x + self.y * self.y + self.z * self.z))

    def normalize(self):
        """Returns copy of the normalized vector """
        mag = self.length()
        return (Vector(self.x / mag, self.y / mag, self.z / mag))

    def __mul__(self, rs):
        """Implements Vector * scalar.  Can then use '*' syntax in multiplying a vector by a scalar rs. """
        x = self.x * rs
        y = self.y * rs
        z = self.z * rs
        return (Vector(x, y, z))

    def __rmul__(self, ls):
        """Implements float * Vector """
        x = self.x * ls
        y = self.y * ls
        z = self.z * ls
        return (Vector(x, y, z))

    def __add__(self, rs):
        """Implements Vector + Vector """
        x = self.x + rs.x
        y = self.y + rs.y
        z = self.z + rs.z
        return (Vector(x, y, z))

    def __sub__(self, rs):
        """Implements Vector - Vector """
        x = self.x - rs.x
        y = self.y - rs.y
        z = self.z - rs.z
        return (Vector(x, y, z))

    def __truediv__(self, rs):
        """Implements Vector / float """
        x = self.x / rs
        y = self.y / rs
        z = self.z / rs
        return (Vector(x, y, z))

    def __div__(self, rs):
        """Implements Vector / float """
        return type(self).__truediv__(self, rs)

    def __imul__(self, rs):
        """Implements Vector *= float """
        self.x *= rs
        self.y *= rs
        self.z *= rs
        return (self)

    def __iadd__(self, rs):
        """Implements Vector += vector """
        self.x += rs.x
        self.y += rs.y
        self.z += rs.z
        return (self)

    def __isub__(self, rs):
        """Implements Vector -= vector """
        self.x -= rs.x
        self.y -= rs.y
        self.z -= rs.z
        return (self)

    def __itruediv__(self, rs):
        """Implements Vector /= float """
        self.x /= rs
        self.y /= rs
        self.z /= rs
        return (self)

    def __idiv__(self, rs):
        """Implements Vector /= float """
        return type(self).__itruediv__(self, rs)

    def create_matrix(self):
        """Converts a Vector into a single-column matrix."""

        column = [self.x, self.y, self.z]
        return(Matrix([[element] for element in column]))  # singleton list

    # Recommend deletion -- better to use a single interface that takes two
    # vectors.
    def dot(self, V2):
        """returns dot product between two vectors """
        return(self.x * V2.x + self.y * V2.y + self.z * V2.z)

    # Recommend deletion in favor of non-method version.
    def cross(self, V2):
        """returns cross product of two vectors """
        x = self.y * V2.z - V1.z * V2.y
        y = self.z * V2.x - V1.x * V2.z
        z = self.x * V2.y - V1.y * V2.x
        return Vector(x, y, z)

    # Replace by separation - RLH
    def angle(self, V2):
        """returns angle between the two vectors in degrees """
        R1 = self.length()
        R2 = V2.length()
        adot = dot(self, V2)
        adot = adot / R1 / R2
        adot = min(1., adot)
        adot = max(-1., adot)
        return acosd(adot)

    # RLH: What do these add?  We're creating methods just to access
    # individual attributes.
    def rx(self): return self.x

    def ry(self): return self.y

    def rz(self): return self.z

    # RLH: Suggest deletion in favor of __str__, which has the advantage that
    # it is called on print.
    def display(self):
        return "[%f, %f, %f]" % (self.x, self.y, self.z)

    # RLH: Not necessary if CelestialVector is used.
    def set_xyz(self, ra, dec):
        """Creates a unit vector from spherical coordinates """
        self.x = cos(dec) * cos(ra)
        self.y = cos(dec) * sin(ra)
        self.z = sin(dec)


NULL_3D_VECTOR = Vector(0, 0, 0)  # this can be reused as needed


class CelestialVector (Vector):
    "Class to encapsulate a unit vector on the celestial sphere."

    def __init__(self, ra=0.0, dec=0.0, frame='eq', degrees=True):
        """Constructor for a celestial vector.

            There are two spherical coordinates, a longitudinal coordinate (called
            right ascension), and a latitudinal coordinate (called declination).
            The RA is defined as the counterclockwise angle from a reference direction
            on the equatorial plane; it ranges from 0-360 degrees.  The DEC is the angle
            between the vector and the equatorial plane; it ranges from -90 to 90 degrees.
            Angles are specified in degrees but represented internally as radians.

            The frame attribute indicates the coordinate frame of the vector, which may be
            'eq' (equatorial, default), 'ec' (ecliptic), or 'gal' (galactic).  In equatorial
            coordinates, the equatorial plane is the celestial equator (extension of the Earth's
            equator) and the reference axis is the vernal equinox.  In ecliptic coordiantes,
            the equatorial plane is the ecliptic (the Earth's orbital plane) and the reference
            axis is usually defined relative to the Sun.  In galactic coordinates, the equatorial
            plane is the plane of the Galaxy.

            The degrees attribute should be True if the RA,DEC inputs are in degrees.
            Otherwise radians is assumed.

            The coordinates "ra" and "dec" may be used in all three systems.  Other names for
            coordinates in different frames may be defined for clarity.

            A CelestialVector is also an ordinary unit vector, with Cartesian coordinates defined
            relative to the equatorial plane."""

        if (degrees):
            ra = math2.D2R * ra
            dec = math2.D2R * dec

        self.ra = ra
        self.dec = dec
        self.frame = frame

        # Initialize standard vector with translated Cartesian coordinates
        Vector.__init__(
            self,
            x=cos(ra) *
            cos(dec),
            y=sin(ra) *
            cos(dec),
            z=sin(dec))

    def __str__(self, verbose=True):
        """Returns a string representation of the vector.  Displays angles in degrees."""
        celest_info = 'CelestialVector: RA: %.3fD, DEC: %.3fD, frame: %s'\
            % (math2.R2D * self.ra, math2.R2D * self.dec, self.frame)

        if (verbose):
            celest_info = celest_info + '\n' + \
                super(CelestialVector, self).__str__()
        return(celest_info)

    def set_eq(self, ra, dec, degrees=False):
        """Modifies a celestial vector with a new RA and DEC.

        degrees = True if units are degrees.  Default is radians."""

        if (degrees):
            ra = math2.D2R * ra
            dec = math2.D2R * dec

        self.ra = ra
        self.dec = dec

        # Update Cartesian coordinates as well.
        super(
            CelestialVector,
            self).set_eq(
            cos(ra) *
            cos(dec),
            sin(ra) *
            cos(dec),
            sin(dec))

    def update_cartesian(self, x=None, y=None, z=None):
        """Modifies a celestial vector by specifying new Cartesian coordinates.

Any subset of the Cartesian coordinates may be specifed."""

        if (x is not None):
            self.x = x
        if (y is not None):
            self.y = y
        if (z is not None):
            self.z = z

        self.ra = atan2(self.y, self.x)  # RA is arctan of y/x
        if (self.ra < 0):  # Make sure RA is positive
            self.ra += 2 * pi

        self.dec = math2.asin2(self.z)  # DEC is arcsin of z

    def rotate_about_axis(self, angle, axis):
        """This rotates a vector about an axis by the specified angle
        by using a rotation matrix.
        A new vector is returned.

        Axis must be 'x', 'y', or 'z'.
        The x-rotation rotates the y-axis toward the z-axis.
        The y-rotation rotates the z-axis toward the x-axis.
        The z-rotation rotates the x-axis toward the y-axis."""

        if (axis == 'x'):
            rot_matrix = Matrix([[1, 0, 0], [0, cos(angle), -sin(angle)],
                                 [0, sin(angle), cos(angle)]])

        elif (axis == 'y'):
            rot_matrix = Matrix([[cos(angle), 0, sin(angle)], [0, 1, 0],
                                 [-sin(angle), 0, cos(angle)]])

        elif (axis == 'z'):
            rot_matrix = Matrix([[cos(angle), -sin(angle), 0],
                                 [sin(angle), cos(angle), 0], [0, 0, 1]])

        else:
            print('Error')
            return

        new_matrix = rot_matrix * self.create_matrix()
        new_vector = new_matrix.column(0)
        result = CelestialVector()  # initialize with Cartesian coordiantes
        result.update_cartesian(
            x=new_vector[0],
            y=new_vector[1],
            z=new_vector[2])
        return(result)

    def rotate_about_eigenaxis(self, angle, eigenaxis):
        """rotates a vector about arbitrary eigenaxis.

        eigenaxis = Vector object (axis about which to rotate).
        angle = angle to rotate by in radians.
        Rotation is counterclockwise looking outward from origin along eigenaxis.
        Function uses rotation matrix from Rodrigues formula.

        Note: This function is more general than rotate_about_axis above and
        could be used in its place.  However, rotate_about_axis is faster and
        clearer when the rotation axis is one of the Cartesian axes."""

        cos_ang = cos(angle)  # Used repeatedly below
        sin_ang = sin(angle)

        # Fill out the Rodrigues rotation matrix
        R11 = cos_ang + eigenaxis.x**2 * (1 - cos_ang)
        R12 = eigenaxis.x * eigenaxis.y * (1 - cos_ang) - eigenaxis.z * sin_ang
        R13 = eigenaxis.x * eigenaxis.z * (1 - cos_ang) + eigenaxis.y * sin_ang
        R21 = eigenaxis.x * eigenaxis.y * (1 - cos_ang) + eigenaxis.z * sin_ang
        R22 = cos_ang + eigenaxis.y**2 * (1 - cos_ang)
        R23 = eigenaxis.y * eigenaxis.z * (1 - cos_ang) - eigenaxis.x * sin_ang
        R31 = eigenaxis.x * eigenaxis.z * (1 - cos_ang) - eigenaxis.y * sin_ang
        R32 = eigenaxis.y * eigenaxis.z * (1 - cos_ang) + eigenaxis.x * sin_ang
        R33 = cos_ang + eigenaxis.z**2 * (1 - cos_ang)

        rot_matrix = Matrix(
            [[R11, R12, R13], [R21, R22, R23], [R31, R32, R33]])
        new_matrix = rot_matrix * self.create_matrix()
        new_vector = new_matrix.column(0)
        result = CelestialVector()  # initialize with Cartesian coordinates
        result.update_cartesian(
            x=new_vector[0],
            y=new_vector[1],
            z=new_vector[2])
        return(result)

    def rotate_using_quaternion(self, angle, eigenaxis):
        """Rotates a vector about arbitrary eigenaxis using quaternion.

        This is an alternative formulation for rotate_about_eigenaxis.
        Interface is the same as rotate_about_eigenaxis."""

        q = Quaternion(eigenaxis, 0.0)

        # Need to negate here because set_values performs a negative rotation
        # quaternion now represents the rotation
        q.set_values(eigenaxis, -angle)
        return(make_celestial_vector(q.cnvrt(self)))

    def transform_frame(self, new_frame):
        """Transforms coordinates between celestial and ecliptic frames

        and returns result as a new CelestialVector.
        If new coordinate frame is the same as the old, a copy of the vector
        is returned."""

        result = None

        # Equatorial to ecliptic: rotate z-axis toward y-axis.
        if ((new_frame == 'ec') and (self.frame == 'eq')):
            result = self.rotate_about_axis(-math2.OBLIQUITY, 'x')

        # Ecliptic to equatorial: rotate y-axis toward z-axis.
        elif ((new_frame == 'eq') and (self.frame == 'ec')):
            result = self.rotate_about_axis(math2.OBLIQUITY, 'x')

        elif ((new_frame == 'gal') and (self.frame == 'eq')):
            # Use formula from Wayne Kinzel's book, adjusted for J2000
            # coordinates.
            b = math2.asin2(cos(self.dec) * cos(NGP.longitude) * cos(self.ra - NGP.latitude)
                            + sin(self.dec) * sin(NGP.longitude))

            l = atan2(sin(self.dec) - sin(b) * sin(NGP.longitude),
                      cos(self.dec) * sin(self.ra - NGP.latitude) * cos(NGP.longitude)) + NGP.anode

            result = CelestialVector(l, b, degrees=False)

        elif ((new_frame == 'eq') and (self.frame == 'gal')):
            l = self.ra  # use l,b notation here for clarity
            b = self.dec

            dec = math2.asin2(cos(b) * cos(NGP.longitude) *
                              sin(l - NGP.anode) + sin(b) * sin(NGP.longitude))

            ra = atan2(cos(b) * cos(l - NGP.anode),
                       sin(b) * cos(NGP.longitude) - cos(b) * sin(NGP.longitude) * sin(l - NGP.anode)) + NGP.latitude

            result = CelestialVector(ra, dec, degrees=False)

        elif (((new_frame == 'gal') and (self.frame == 'ec')) or ((new_frame == 'ec') and (self.frame == 'gal'))):
            print(
                "Error: Direct conversion between ecliptic and galactic coordinates not supported yet")

        elif (new_frame != self.frame):
            print("Error: unrecognized coordinate frame.")

        # If there was an error, return a copy of the initial vector.
        if (result is None):
            result = CelestialVector(self.ra, self.dec, self.frame, False)

        else:
            result.frame = new_frame  # record new frame

        return (result)

    def rotate_by_posang(self, pa):
        """Returns the vector that results from rotating the self vector

            counterclockwise from the North projection onto the plane
            orthogonal to that vector by the specified position angle
        (in radians).
        See "V3-axis Position Angle", John Isaacs, May 2003 for
        further discussion."""

        x_coord = -cos(self.ra) * sin(self.dec) * \
            cos(pa) - sin(self.ra) * sin(pa)
        y_coord = -sin(self.ra) * sin(self.dec) * \
            cos(pa) + cos(self.ra) * sin(pa)
        z_coord = cos(self.dec) * cos(pa)
        result = CelestialVector()
        result.update_cartesian(x_coord, y_coord, z_coord)
        return(result)

    def position_angle(self, v):
        """Returns the position angle of v at the self vector, in radians.

        v is an arbitrary vector that should be a CelestialVector object.
        The position angle is the angle between the North vector on the
        plane orthogonal to the self vector and the projection of v onto
        that plane, defined counterclockwise.
        See "V3-axis Position Angle", John Isaacs, May 2003 for
        further discussion."""

        y_coord = cos(v.dec) * sin(v.ra - self.ra)
        x_coord = sin(v.dec) * cos(self.dec) - cos(v.dec) * \
            sin(self.dec) * cos(v.ra - self.ra)
        pa = atan2(y_coord, x_coord)

        if (pa < 0):
            pa += (2 * pi)  # PA has range 0-360 degrees

        return(pa)


class Attitude (CelestialVector):
    "Defines an Observatory attitude by adding a position angle."""

    def __init__(self, ra=0.0, dec=0.0, pa=0.0, frame='eq', degrees=True):
        """Constructor for an Attitude.

        pa = position_angle in degrees(default) or radians if degrees=False is specified.
        Other arguments are the same as with CelestialVector."""

        super(
            Attitude,
            self).__init__(
            ra=ra,
            dec=dec,
            frame=frame,
            degrees=degrees)

        if (degrees):  # convert into radians
            pa = math2.D2R * pa

        self.pa = pa

    def __str__(self, verbose=True):
        """Returns a string representation of the attitude.

        verbose (optional) = flag indicating whether detailed Vector information should be included."""

        att_info = 'Attitude: PA: %.3fD' % (math2.R2D * self.pa)
        att_info = att_info + '\n' + super(Attitude, self).__str__(verbose)
        return(att_info)


# Functions that operate on vectors but are not methods.
def dot(v1, v2):
    """returns dot product between two vectors, non class member """

    return(v1.x * v2.x + v1.y * v2.y + v1.z * v2.z)


def cross(v1, v2):
    """returns cross product between two vectors, non class member """
    x = v1.y * v2.z - v1.z * v2.y
    y = v1.z * v2.x - v1.x * v2.z
    z = v1.x * v2.y - v1.y * v2.x
    return Vector(x, y, z)


def separation(v1, v2, norm=False):
    """Returns angle between two unit vectors in radians.

    The angle between two normalized vectors is the arc-cosine of the dot product.
    Unless the norm attribute is set to True, it is assumed the vectors are
    already normalized (for performance)."""

    if (norm):
        v1 = v1.normalize()
        v2 = v2.normalize()

    separation = math2.acos2(dot(v1, v2))

    # For very small angles, cos and acos behave poorly as the cosine of a very small angle is
    # interpreted as 1.0.  Therefore, recompute using the cross product if the
    # result is less than 1 degree.
    if (separation < math2.D2R):
        vcross = cross(v1, v2)
        separation = math2.asin2(vcross.length())

    return(separation)


def ra_delta(v1, v2):
    """Returns difference in right ascension between two CelestialVectors."""

    delta_ra = v1.ra - v2.ra

    # Check for zero crossings.  If the difference exceeds 180 degrees, adjust
    # by 360 in opposite direction.

    if (delta_ra < -pi):
        delta_ra = delta_ra + 2 * pi
    elif (delta_ra > pi):
        delta_ra = delta_ra - 2 * pi

    return(delta_ra)


def ra_separation(v1, v2):
    """Returns separation in right ascension between two CelestialVectors.
    This is accurate only if the difference in declination is small.

    |sep| = DELTA-RA cos DEC."""

    delta_ra = ra_delta(v1, v2)
    dec = math2.avg2(v1.dec, v2.dec)  # use average of the two declinations.
    return(delta_ra * cos(dec))


def dec_separation(v1, v2):
    """Returns difference in declination between two CelestialVectors."""

    return(v1.dec - v2.dec)  # simply take the difference in declination


def make_celestial_vector(v):
    """Takes a Vector object and creates an equivalent CelestialVector.

    Input vector v must be a unit vector."""

    result = CelestialVector()
    result.update_cartesian(v.x, v.y, v.z)
    return(result)


def projection(v, axis):
    """Returns projection of vector v on plane normal to axis.

    First take cross-product of v and the axis and normalize it.
    Then cross the axis with the result and return a CelestialVector.
    See http://www.euclideanspace.com/maths/geometry/elements/plane/lineOnPlane/index.htm."""

    return(make_celestial_vector(cross(axis, (cross(v, axis)).normalize())))


def pos_V_to_ra_dec(V):
    """Returns tuple of spherical angles from unit direction Vector """
    ra = atan2d(V.y, V.x)
    V.z = min(1., V.z)
    V.z = max(-1., V.z)
    dec = asind(V.z)
    if ra < 0.:
        ra += 360.
    return(ra, dec)

# RLH: Recommend replacement by separation.


def angle(V1, V2):
    """returns angle between two vectors in degrees, non class member """
    R1 = V1.length()
    R2 = V2.length()
    adot = dot(V1, V2)
    adot = adot / R1 / R2
    adot = min(1., adot)
    adot = max(-1., adot)
    return acosd(adot)


def vel_ab(U, Vel):
    """ Takes a unit vector and a velocity vector(km/s) and returns a unit vector modidifed by the velocity abberation."""
    c = 2.9979e5  # speed of light in km/s
    Beta = Vel * (1. / c)
    rgamma = math.sqrt(1. - dot(Beta, Beta))  # This is 1/gamma
    ubeta = dot(U, Beta)
    return (U * rgamma + Beta * (1. + (1. - rgamma) *
                                 ubeta / dot(Beta, Beta))) * (1. / (1. + ubeta))



#! /usr/bin/env python
# Module ephemeris.py
from __future__ import absolute_import, division, print_function

import sys
import time

from math import asin, atan2, cos, sin, pi
from . import astro_funcx as astro_func
from . import quaternionx
from . import rotationsx

D2R = pi / 180.  # degrees to radians
R2D = 180. / pi  # radians to degrees
PI2 = 2. * pi  # 2 pi


def unit_limit(x): return min(max(-1., x), 1.)  # forces value to be in [-1,1]


MIN_SUN_ANGLE = 84.8 * D2R  # minimum Sun angle, in radians
MAX_SUN_ANGLE = 135.0 * D2R  # maximum Sun angle, in radians
# pad away from Sun angle limits when constructing safe attitude
SUN_ANGLE_PAD = 0.5 * D2R

obliquity_of_the_ecliptic = -23.439291  # At J2000 equinox
obliquity_of_the_ecliptic *= D2R
Qecl2eci = quaternionx.QX(obliquity_of_the_ecliptic)


class Ephemeris:
    def __init__(self, afile, cnvrt=False):
        """Eph constructor, cnvrt True converts into Ecliptic frame """
        if cnvrt:
            print("Using Ecliptic Coordinates")
        else:
            print("Using Equatorial Coordinates")
        self.datelist = []
        self.xlist = []
        self.ylist = []
        self.zlist = []
        self.amin = 0.
        self.amax = 0.
        aV = rotationsx.Vector(0., 0., 0.)
        with open(afile, 'r') as afp:
            fin = afp.readlines()
            if "l2_halo_FDF_060619.trh" in afile:
                ascale = 0.001
            else:
                ascale = 1.0
            if "horizons_EM_L2" in afile:
                not_there = True
                istart = 0
                while fin[istart][:5] != "$$SOE":
                    if fin[istart].find('Center body name:') > - \
                            1:  # Checks that the Sun is the central body!
                        if fin[istart].find('Sun') > -1:
                            not_there = False
                        else:
                            print(fin[istart])
                    istart += 1
                istart += 1
                if not_there:
                    print(
                        "This ephemeris does not use the Sun as the center body.  It should not be used.")
                    exit(-1)

                while fin[istart][:5] != "$$EOE":
                    item = fin[istart].strip()
                    item = item.split(',')
                    # represent dates as mjds
                    adate = float(item[0]) - 2400000.5
                    x = float(item[2]) * ascale
                    y = float(item[3]) * ascale
                    z = float(item[4]) * ascale
                    if cnvrt:
                        aV.set_eq(x, y, z)
                        ll = aV.length()
                        aV = aV / ll
                        aV = Qecl2eci.inv_cnvrt(aV)
                        aV = aV * ll
                        x = aV.rx()
                        y = aV.ry()
                        z = aV.rz()
                    self.datelist.append(adate)
                    self.xlist.append(x)
                    self.ylist.append(y)
                    self.zlist.append(z)
                    if self.amin == 0.:
                        self.amin = adate
                    istart += 1
            else:
                for item in fin[2:]:
                    item = fin[istart].strip()
                    item = item.split(',')
                    adate = time2.mjd_from_string(
                        item[0])  # represent dates as mjds
                    x = float(item[1]) * ascale
                    y = float(item[2]) * ascale
                    z = float(item[3]) * ascale
                    if cnvrt:
                        aV.set_eq(x, y, z)
                        ll = aV.length()
                        aV = aV / ll
                        aV = Qecl2eci.inv_cnvrt(aV)
                        aV = aV * ll
                        x = aV.rx()
                        y = aV.ry()
                        z = aV.rz()
                    self.datelist.append(adate)
                    self.xlist.append(x)
                    self.ylist.append(y)
                    self.zlist.append(z)
                    if self.amin == 0.:
                        self.amin = adate
            self.amax = adate
            # yp = spline(xa,ya,0.,0.)
            # Saving spline parameters
            # self.xlistp = spline(self.datelist,self.xlist,1.e31,1.e31)
            # self.ylistp = spline(self.datelist,self.ylist,1.e31,1.e31)
            # self.zlistp = spline(self.datelist,self.zlist,1.e31,1.e31)
            del fin
            # print()
            # len(self.datelist),len(self.xlist),len(self.ylist),len(self.zlist)

    def report_ephemeris(self, limit=100000, pathname=None):
        """Prints a formatted report of the ephemeris.

        If a limit is specified, no more than the maximum number of records are reported.
        pathname = optional path to a file to hold the report."""

        num_to_report = min(limit, len(self.datelist))

        if (pathname):
            dest = open(pathname, 'w')
            print(('#Generated %s\n' % (time.ctime())), file=dest)
        else:
            dest = sys.stdout  # defaults to standard output

        print(('%17s  %14s  %14s  %14s\n' %
               ('DATE      ', 'X (KM)   ', 'Y (KM)   ', 'Z (KM)   ')), file=dest)

        for num in range(num_to_report):
            date = self.datelist[num]
            x = self.xlist[num]
            y = self.ylist[num]
            z = self.zlist[num]

            print(('%17s  %14.3f  %14.3f  %14.3f' %
                   (time2.display_date(date), x, y, z)), file=dest)

        if (pathname):
            dest.close()  # Clean up

    def pos(self, adate):
        cal_days = adate - self.datelist[0]
        indx = int(cal_days)
        frac = cal_days - indx
        x = (self.xlist[indx + 1] - self.xlist[indx]) * frac + self.xlist[indx]
        y = (self.ylist[indx + 1] - self.ylist[indx]) * frac + self.ylist[indx]
        z = (self.zlist[indx + 1] - self.zlist[indx]) * frac + self.zlist[indx]
        return rotationsx.Vector(x, y, z)
        # alower = float(int(adate - 0.5)) + 0.5
        # if alower>= self.amin and adate>= self.amin and adate<=self.amax:
        #     x = spline_interp(adate,self.datelist,self.xlist)
        #     y = spline_interp(adate,self.datelist,self.ylist)
        #     z = spline_interp(adate,self.datelist,self.zlist)
        # x = splint(self.datelist,self.xlist,self.xlistp,adate)
        # y = splint(self.datelist,self.ylist,self.ylistp,adate)
        # z = splint(self.datelist,self.zlist,self.zlistp,adate)
        # splint(xa,ya,yp,x)
        # return Vector(x,y,z)

    def Vsun_pos(self, adate):
        Vsun = -1. * self.pos(adate)
        Vsun = Vsun / Vsun.length()
        return Vsun

    def sun_pos(self, adate):
        Vsun = -1. * self.pos(adate)
        Vsun = Vsun / Vsun.length()
        coord2 = asin(quaternionx.unit_limit(Vsun.z))
        coord1 = atan2(Vsun.y, Vsun.x)
        if coord1 < 0.:
            coord1 += quaternionx.PI2
        return (coord1, coord2)

    def normal_pa(self, adate, tgt_c1, tgt_c2):
        (sun_c1, sun_c2) = self.sun_pos(adate)
        sun_pa = astro_func.pa(tgt_c1, tgt_c2, sun_c1, sun_c2)
        V3_pa = sun_pa + pi  # We want -V3 pointed towards sun.
        if V3_pa < 0.:
            V3_pa += quaternionx.PI2
        if V3_pa >= quaternionx.PI2:
            V3_pa -= quaternionx.PI2
        return V3_pa

    def is_valid(self, date, coord_1, coord_2, V3pa):
        """Indicates whether an attitude is valid at a given date."""

        # First check that the date is within the time interval of the
        # ephemeris.
        if ((date < self.amin) or (date > self.amax)):
            return False

        (sun_1, sun_2) = self.sun_pos(date)
        d = astro_func.dist(coord_1, coord_2, sun_1, sun_2)
        vehicle_pitch = pi / 2 - d  # see JI memo from May 2006
        # sun pitch is always equal or greater than sun angle (V1 to sun)
        if (d < MIN_SUN_ANGLE or d > MAX_SUN_ANGLE):
            return False
        pa = astro_func.pa(coord_1, coord_2, sun_1, sun_2) + pi
        roll = acos(cos(V3pa - pa))
        sun_roll = asin(sin(roll) * cos(vehicle_pitch))
        if (abs(sun_roll) <= 5.2 * quaternionx.quaternionx.D2R):
            sun_pitch = atan2(tan(vehicle_pitch), cos(roll))
            if (sun_pitch <= 5.0 * quaternionx.quaternionx.D2R and sun_pitch >= -44.8 * quaternionx.quaternionx.D2R):
                return True
        return False

    def in_FOR(self, adate, coord_1, coord_2):
        (sun_1, sun_2) = self.sun_pos(adate)
        d = astro_func.dist(coord_1, coord_2, sun_1, sun_2)
        # print d*R2D
        # 90 - sun pitch is always equal or greater than sun angle (V1 to sun)
        if (d < MIN_SUN_ANGLE or d > MAX_SUN_ANGLE):
            return False
        return True

    # in and out of FOR, assumes only one "root" in interval
    def bisect_by_FOR(self, in_date, out_date, coord_1, coord_2):
        delta_days = 200.
        mid_date = (in_date + out_date) / 2.
        while delta_days > 0.000001:
            (sun_1, sun_2) = self.sun_pos(mid_date)
            d = astro_func.dist(coord_1, coord_2, sun_1, sun_2)
            if (d > MAX_SUN_ANGLE or d < MIN_SUN_ANGLE):
                out_date = mid_date
            else:
                in_date = mid_date
            mid_date = (in_date + out_date) / 2.
            delta_days = abs(in_date - out_date) / 2.
            # print "UU", mid_date
        if in_date > out_date:  # ensure returned date always in FOR
            mid_date = mid_date + 0.000001
        else:
            mid_date = mid_date - 0.000001
        return mid_date

    # in and out of FOR, assumes only one "root" in interval
    def bisect_by_attitude(self, in_date, out_date, coord_1, coord_2, pa):
        icount = 0
        delta_days = 200.
        mid_date = (in_date + out_date) / 2.
        # print "bisect >",in_date,out_date,abs(in_date-out_date )
        while delta_days > 0.000001:
            if self.is_valid(mid_date, coord_1, coord_2, pa):
                in_date = mid_date
            else:
                out_date = mid_date
            mid_date = (in_date + out_date) / 2.
            delta_days = abs(in_date - out_date) / 2.
            # print "UU", mid_date
            icount = icount + 1
        # print " bisected >",icount
        return mid_date



#!/usr/bin/env python
# encoding: utf-8
from __future__ import absolute_import, division, print_function

import json
import os

from .. import PKG_DATA_DIR, CONFIG_DIR, CONFIG_FILE
from ..footprints import footprints
from ..defaults import default_config
from ..plot_timeline import plottimeline

try:
    from Tkinter import (Tk, Button, Entry, Label, OptionMenu, StringVar, \
                         Toplevel, TRUE, FALSE)
    from tkFileDialog import askopenfilename
except ImportError as e:
    from tkinter.filedialog import askopenfilename
    from tkinter import (Tk, Button, Entry, Label, OptionMenu, StringVar, \
                         Toplevel, TRUE, FALSE)

from PIL import ImageTk


class Config(object):
    def __init__(self, filename=CONFIG_FILE):
        self.filename = os.path.abspath(filename)
        self.exists = False
        self.data = {}

        self._check()
        self._parse()

    def __getitem__(self, x):
        if x in self.data:
            return self.data[x]

    def __setitem__(self, k, v):
        self.data[k] = v

    def _check(self):
        if CONFIG_DIR in CONFIG_FILE:
            if not os.path.exists(CONFIG_DIR):
                os.makedirs(CONFIG_DIR, mode=0o0755)

        if os.path.exists(self.filename):
            self.exists = True

    def _parse(self):
        # If we already have a config file, but haven't populated the data,
        # then do so:
        if self.exists and not self.data:
            with open(self.filename, 'r') as cfg:
                self.data = json.load(cfg)

        # Use pre-determined configuration, but generate a new user-config
        # based on it:
        elif not self.exists:
            self.data = default_config
            self.commit()

    def commit(self):
        with open(self.filename, 'w+') as cfg:
            json.dump(self.data, cfg, sort_keys=True, indent=4)


class CreateToolTip(object):
    """
    create a tooltip for a given widget
    """

    def __init__(self, widget, text='widget info'):
        self.waittime = 500  # miliseconds
        self.wraplength = 180  # pixels
        self.widget = widget
        self.text = text
        self.widget.bind("<Enter>", self.enter)
        self.widget.bind("<Leave>", self.leave)
        self.widget.bind("<ButtonPress>", self.leave)
        self.id = None
        self.tw = None

    def enter(self, event=None):
        self.schedule()

    def leave(self, event=None):
        self.unschedule()
        self.hidetip()

    def schedule(self):
        self.unschedule()
        self.id = self.widget.after(self.waittime, self.showtip)

    def unschedule(self):
        id = self.id
        self.id = None
        if id:
            self.widget.after_cancel(id)

    def showtip(self, event=None):
        x = y = 0
        x, y, cx, cy = self.widget.bbox("insert")
        x += self.widget.winfo_rootx() + 25
        y += self.widget.winfo_rooty() + 20
        # creates a toplevel window
        self.tw = Toplevel(self.widget)
        # Leaves only the label and removes the app window
        self.tw.wm_overrideredirect(True)
        self.tw.wm_geometry("+%d+%d" % (x, y))
        label = Label(self.tw, text=self.text, justify='left',
                      background="#ffffff", relief='solid', borderwidth=0,
                      wraplength=self.wraplength)
        label.pack(ipadx=1)

    def hidetip(self):
        tw = self.tw
        self.tw = None
        if tw:
            tw.destroy()


class FootprintsUI(object):
    def __init__(self, master):
        self.config = Config()
        self.master = master
        self.master.resizable(width=FALSE, height=FALSE)
        self.master.title('NIRspec Observation Visualization Tool')
        self.background = ImageTk.PhotoImage(file=os.path.join(PKG_DATA_DIR, "back-08.png"))

        w = self.background.width()
        h = self.background.height()

        # position coordinates of root 'upper left corner'
        x = 0
        y = 0

        # make the root window the size of the image
        self.master.geometry("%dx%d+%d+%d" % (w, h, x, y))
        # root has no image argument, so use a label as a panel
        self.panel1 = Label(self.master, image=self.background)
        # panel1.pack(side='top', fill='both', expand='Yes')
        self.panel1.place(x=0, y=0)
        # panel1.image = image1


        # assigning values to colors
        self.colmsaVar = StringVar()
        self.colmsaVar.set(self.config['color_msa'])
        self.colshortVar = StringVar()
        self.colshortVar.set(self.config['color_short'])
        self.collongVar = StringVar()
        self.collongVar.set(self.config['color_long'])

        self.readfitsimageVar = StringVar()
        self.readfitsimageVar.set(self.config['readfitsimage'])
      #  print(readfitsimageVar)


        self.labplotsources = Label(self.master, text=" Catalog on ?")
        self.labplotsources.place(relx=0.44, rely=0.26, anchor="w")
        self.plotsourcesVar = StringVar()
        self.plotsourcesVar.set(self.config['plot_src'])
        self.pt = OptionMenu(self.master, self.plotsourcesVar, 'Yes', 'No')
        self.pt.place(relx=0.29, rely=0.26, anchor="w")

        self.labplotmsa = Label(self.master, text="MSA footprint on ?")
        self.labplotmsa.place(relx=0.65, rely=0.38, anchor="w")
        self.plotmsaVar = StringVar()
        self.plotmsaVar.set(self.config['plot_msa'])
        self.pt = OptionMenu(self.master, self.plotmsaVar, 'Yes', 'No')
        self.pt.place(relx=0.35, rely=0.38, anchor="w")

        self.colmsaVar = StringVar()
        self.colmsaVar.set(self.config['color_msa'])
        self.colmsa = OptionMenu(self.master, self.colmsaVar, 'Red', 'Cyan', 'Blue', 'Yellow','Green')
        self.colmsa.config(width=9)
        self.colmsa.place(relx=0.49, rely=0.38, anchor="w")


        self.labramsa = Label(self.master, text="RA center of MSA")
        self.labramsa.place(relx=0.65, rely=0.42, anchor="w")
        self.ramsaVar = StringVar()
        self.ramsaVar.set(self.config['ra_nirspec'])
        self.ramsa = Entry(self.master, textvariable=self.ramsaVar)
        self.ramsa.place(relx=0.35, rely=0.42, anchor="w")

        self.ramsa_ttp = CreateToolTip(self.ramsa,
                                       'Enter RA value.\nxxx.xxxx (degrees)\nhh mm ss.sss')

        self.labdecmsa = Label(self.master, text="DEC center of MSA")
        self.labdecmsa.place(relx=0.65, rely=0.46, anchor="w")
        self.decmsaVar = StringVar()
        self.decmsaVar.set(self.config['dec_nirspec'])
        self.decmsa = Entry(self.master, textvariable=self.decmsaVar)
        self.decmsa.place(relx=0.35, rely=0.46, anchor="w")
        self.decmsa_ttp = CreateToolTip(self.decmsa,
                                        'Enter DEC value.\nxxx.xxxx (degrees)\nhh mm ss.sss')

        self.labthmsa = Label(self.master, text="MSA aperture PA")
        self.labthmsa.place(relx=0.65, rely=0.50, anchor="w")
        self.thmsaVar = StringVar()
        self.thmsaVar.set(self.config['theta_nirspec'])
        self.thmsa = Entry(self.master, textvariable=self.thmsaVar)
        self.thmsa.place(relx=0.35, rely=0.50, anchor="w")
        self.thmsa_ttp = CreateToolTip(self.thmsa,
                                       'Enter MSA Aperture PA value in degrees')

        self.labplotshort = Label(self.master, text="Short channel on ?")
        self.labplotshort.place(relx=0.65, rely=0.61, anchor="w")
        self.plotshortVar = StringVar()
        self.plotshortVar.set(self.config['plot_short'])
        self.pt = OptionMenu(self.master, self.plotshortVar, 'Yes', 'No')
        self.pt.config(width=8)
        self.pt.place(relx=0.35, rely=0.61, anchor="w")

        self.labplotlong = Label(self.master, text="Long channel on ?")
        self.labplotlong.place(relx=0.65, rely=0.65, anchor="w")
        self.plotlongVar = StringVar()
        self.plotlongVar.set(self.config['plot_long'])
        self.pt = OptionMenu(self.master, self.plotlongVar, 'Yes', 'No')
        self.pt.config(width=8)
        self.pt.place(relx=0.35, rely=0.65, anchor="w")

        self.labranrcm = Label(self.master, text="RA center of NIRCam")
        self.labranrcm.place(relx=0.65, rely=0.69, anchor="w")
        self.ranrcmVar = StringVar()
        self.ranrcmVar.set(self.config['ra_nircam'])
        self.ranrcm = Entry(self.master, textvariable=self.ranrcmVar)
        self.ranrcm.place(relx=0.35, rely=0.69, anchor="w")
        self.ranrcm_ttp = CreateToolTip(self.ranrcm,
                                        'Enter RA value.\nxxx.xxxx (degrees)\nhh mm ss.sss')

        self.labdecnrcm = Label(self.master, text="DEC center of NIRCam")
        self.labdecnrcm.place(relx=0.65, rely=0.73, anchor="w")
        self.decnrcmVar = StringVar()
        self.decnrcmVar.set(self.config['dec_nircam'])
        self.decnrcm = Entry(self.master, textvariable=self.decnrcmVar)
        self.decnrcm.place(relx=0.35, rely=0.73, anchor="w")
        self.decnrcm_ttp = CreateToolTip(self.decnrcm,
                                         'Enter DEC value.\nxxx.xxxx (degrees)\nhh mm ss.sss')
        self.labthnrcm = Label(self.master, text="NIRCam aperture PA")
        self.labthnrcm.place(relx=0.65, rely=0.77, anchor="w")
        self.thnrcmVar = StringVar()
        self.thnrcmVar.set(self.config['theta_nircam'])
        self.thnrcm = Entry(self.master, textvariable=self.thnrcmVar)
        self.thnrcm.place(relx=0.35, rely=0.77, anchor="w")
        self.thnrcm_ttp = CreateToolTip(self.thnrcm,
                                        'Enter NIRCam Aperture PA value in degrees')

        self.labdither = Label(self.master, text="NIRCam dither pattern")
        self.labdither.place(relx=0.02, rely=0.81, anchor="w")
        self.ditherVar = StringVar()
        self.ditherVar.set(self.config['dither'])
        self.pt = OptionMenu(self.master, self.ditherVar, 'None', 'FULL3', 'FULL3TIGHT', 'FULL6', '8NIRSPEC')
        self.pt.place(relx=0.35, rely=0.81, anchor="w")

        self.labmosaic = Label(self.master, text="NIRCam mosaic")
        self.labmosaic.place(relx=0.02, rely=0.87, anchor="w")
        self.mosaicVar = StringVar()
        self.mosaicVar.set(self.config['mosaic'])
        self.pt = OptionMenu(self.master, self.mosaicVar, 'No', 'Yes')
        self.pt.place(relx=0.35, rely=0.87, anchor="w")

        self.laboffhor = Label(self.master, text="Offset")
        self.laboffhor.place(relx=0.50, rely=0.85, anchor="w")
        self.offhorVar = StringVar()
        self.offhorVar.set(self.config['off_h'])
        self.offhor = Entry(self.master, textvariable=self.offhorVar)
        self.offhor.place(relx=0.65, rely=0.85, anchor="w")
        self.offhor_ttp = CreateToolTip(self.offhor,
                                        'Enter NIRCam offset in arcsec')

        self.laboffver = Label(self.master, text="Offset")
        self.laboffver.place(relx=0.50, rely=0.89, anchor="w")
        self.offverVar = StringVar()
        self.offverVar.set(self.config['off_v'])
        self.offver = Entry(self.master, textvariable=self.offverVar)
        self.offver.place(relx=0.65, rely=0.89, anchor="w")
        self.offver_ttp = CreateToolTip(self.offver,
                                        'Enter NIRCam offset in arcsec')




        self.ptVar = StringVar()
        self.ptVar.set('footprints')

        self.b5 = Button(self.master, text=" Select File ", command=self.readfitsfilename)
        self.b5.place(relx=0.43, rely=0.12, anchor="w")

        self.fileVar = StringVar()
        self.fileVar.set(self.config['fits_name'])
        self.filevalue = Entry(self.master, textvariable=self.fileVar, width=40, justify='left')
        self.filevalue.place(relx=0.02, rely=0.15, anchor="w")

        self.b6 = Button(self.master, text=" Select File ", command=self.readcataloguename)
        self.b6.place(relx=0.43, rely=0.295, anchor="w")
        self.catVar = StringVar()
        self.catVar.set(self.config['cat_name'])
        self.catvalue = Entry(self.master, textvariable=self.catVar, width=28, justify='left')
        self.catvalue.place(relx=0.02, rely=0.295, anchor="w")

        self.b7 = Button(self.master, text="View Timeline", command=self.maketimeline)
        self.b7.place(relx=0.6, rely=0.96, anchor="w")
        self.b3 = Button(self.master, text=" Quit ", command=self.quit)
        self.b3.place(relx=0.2, rely=0.96, anchor="w")

        self.b4 = Button(self.master, text=" Display ", command=self.makefootprints)
        self.b4.place(relx=0.8, rely=0.96, anchor="w")


        self.outdirVar = StringVar()
        self.outdirVar.set(self.config['out_dir'])
        self.outdirvalue = Entry(self.master, textvariable=self.outdirVar, width=40, justify='left')
        self.outdirvalue.place(relx=0.02, rely=0.219, anchor="w")
        #self.laboutdir = Label(self.master, text="Output directory")
        #self.laboutdir.place(relx=0.02, rely=0.25, anchor="w")
      

        self.colshortVar = StringVar()
        self.colshortVar.set(self.config['color_short'])
        self.colshort = OptionMenu(self.master, self.colshortVar, 'Green', 'Cyan','Blue','Yellow','Red')
        self.colshort.config(width=9)
        self.colshort.place(relx=0.49, rely=0.61, anchor="w")

        self.collongVar = StringVar()
        self.collongVar.set(self.config['color_long'])
        self.collong = OptionMenu(self.master, self.collongVar, 'Blue', 'Cyan','Green','Yellow','Red')
        self.collong.config(width=9)
        self.collong.place(relx=0.49, rely=0.65, anchor="w")

 

        self.labcmap= Label(self.master, text="Colour Map")
        self.labcmap.place(relx=0.65, rely=0.18, anchor="w")
        self.ds9cmapVar = StringVar()
        self.ds9cmapVar.set(self.config['cmap'])
        self.ds9cmap = OptionMenu(self.master, self.ds9cmapVar, 'grey', 'red', 'green', 'blue', 'heat')
        self.ds9cmap.place(relx=0.80, rely=0.18, anchor="w")
        self.ds9cmap.config(width=10)


        self.labscale = Label(self.master, text="Scale")
        self.labscale.place(relx=0.65, rely=0.21, anchor="w")
        self.ds9scaleVar = StringVar()
        self.ds9scaleVar.set(self.config['scale'])
        self.ds9scale = OptionMenu(self.master, self.ds9scaleVar, 'log', 'linear', 'power', 'sqrt', 'zscale', 'minmax')
        self.ds9scale.place(relx=0.80, rely=0.21, anchor="w")
        self.ds9scale.config(width=10)

        self.ds9limmin = Label(self.master, text="Low")
        self.ds9limmin.place(relx=0.65, rely=0.24, anchor="w")
        self.ds9limminVar = StringVar()
        self.ds9limminVar.set(self.config['lim_min'])
        self.ds9limmin = Entry(self.master, textvariable=self.ds9limminVar)
        self.ds9limmin.place(relx=0.80, rely=0.24, anchor="w")
        self.ds9limmin.config(width=10)
       
        self.ds9limmax = Label(self.master, text="High")
        self.ds9limmax.place(relx=0.65, rely=0.27, anchor="w")
        self.ds9limmaxVar = StringVar()
        self.ds9limmaxVar.set(self.config['lim_max'])
        self.ds9limmax = Entry(self.master, textvariable=self.ds9limmaxVar)
        self.ds9limmax.place(relx=0.80, rely=0.27, anchor="w")
        self.ds9limmax.config(width=10)
  

    def quit(self):
        self.config['fits_name'] = self.fileVar.get()
        self.config['cat_name'] = self.catVar.get()
        self.config['plot_long'] = self.plotlongVar.get()
        self.config['plot_short'] = self.plotshortVar.get()
        self.config['plot_msa'] = self.plotmsaVar.get()
        self.config['plot_src'] = self.plotsourcesVar.get()
        self.config['ra_nircam'] = self.ranrcmVar.get()
        self.config['dec_nircam'] = self.decnrcmVar.get()
        self.config['theta_nircam'] = float(self.thnrcmVar.get())
        self.config['dither'] = self.ditherVar.get()
        self.config['ra_nirspec'] = self.ramsaVar.get()
        self.config['dec_nirspec'] = self.decmsaVar.get()
        self.config['theta_nirspec'] = float(self.thmsaVar.get())
        self.config['mosaic'] = self.mosaicVar.get()
        self.config['off_h'] = float(self.offhorVar.get())
        self.config['off_v'] = float(self.offverVar.get())
        self.config['color_msa'] = self.colmsaVar.get()
        self.config['color_short'] = self.colshortVar.get()
        self.config['color_long'] = self.collongVar.get()
        self.config['cmap'] = self.ds9cmapVar.get()
        self.config['lim_min'] = float(self.ds9limminVar.get())
        self.config['lim_max'] = float(self.ds9limmaxVar.get())
        self.config['scale'] = self.ds9scaleVar.get()
        
        self.config.commit()
        self.master.quit()

    def makefootprints(self):
        if self.ptVar.get() == 'footprints':
            #print(self.catVar.get())
            footprints(self.fileVar.get(),
                       self.catVar.get(),
                       self.plotlongVar.get(),
                       self.plotshortVar.get(),
                       self.plotmsaVar.get(),
                       self.plotsourcesVar.get(),
                       self.ranrcmVar.get(),
                       self.decnrcmVar.get(),
                       float(self.thnrcmVar.get()),
                       self.ditherVar.get(),
                       self.ramsaVar.get(),
                       self.decmsaVar.get(),
                       float(self.thmsaVar.get()),
                       self.mosaicVar.get(),
                       self.offhorVar.get(),
                       self.offverVar.get(),
                       self.colmsaVar.get(),
                       self.colshortVar.get(),
                       self.collongVar.get(),
                       self.ds9cmapVar.get(),
                       self.ds9limminVar.get(),
                       self.ds9limmaxVar.get(),
                       self.ds9scaleVar.get(),
                       self.outdirVar.get())
                       #self.readfitsimageVar.get())

    def readcataloguename(self):
        Tk().withdraw()  # we don't want a full GUI, so keep the root window from appearing
        # show an "Open" dialog box and return the path to the selected file
        filename = askopenfilename(initialdir=os.path.abspath(os.curdir),
                                   title='Select RADEC file',
                                   filetypes=(('RADEC files', '*.radec'),
                                              ('all files', '*.*')))

        #print(filename)
        self.catVar.set(filename)

        catvalue = Entry(self.master, textvariable=self.catVar, width=28, justify='right')
        catvalue.place(relx=0.02, rely=0.295, anchor="w")

    def readfitsfilename(self):
        #readfitsimage =True     # set variable to read new image

        Tk().withdraw()  # we don't want a full GUI, so keep the root window from appearing
        # show an "Open" dialog box and return the path to the selected file
        filename = askopenfilename(initialdir=os.path.abspath(os.curdir),
                                   title='Select FITS file',
                                   filetypes=(('FITS files', '*.fits'),
                                              ('all files', '*.*')))
        #print(filename)
        self.fileVar.set(filename)

        filevalue = Entry(self.master, textvariable=self.fileVar, width=40, justify='right')
        filevalue.place(relx=0.02, rely=0.15, anchor="w")

    def maketimeline(self):
        #print(self.ramsaVar.get())
        plottimeline(self.ramsaVar.get(),
                     self.decmsaVar.get(),
                     float(self.thmsaVar.get()),
                     self.outdirVar.get())


def main():
    root = Tk()
    FootprintsUI(root)
    root.mainloop()


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages


setup(
    name='jwst_footprints',
    version='2.4.1',
    author='Leonardo Ubeda',
    author_email='lubeda@stsci.edu',
    description='The JWST NIRSpec Observation Visualization Tool is a Python application that provides a simultaneous view of both NIRSpec and NIRCam fields of view on a given sky position, for assistance in planning NIRCam pre-imaging for NIRSpec.',
    url='https://github.com/spacetelescope/jwst_footprints',
    classifiers=[
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: Astronomy',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
        'astropy',
        'matplotlib',
        'numpy',
        'scipy',
        'pillow',
        'pyds9'
    ],

    packages=find_packages(),
    package_data={
        '': [
            'data/*',
        ]
    },
    entry_points={
        'gui_scripts': [
            'jwst_footprints=jwst_footprints.gui.footprints:main',
        ],
    },
)



#! /usr/bin/env python
from __future__ import absolute_import, division, print_function
# quaternion module

"""Version 4 September 9, 2010 WMK
Flipped sign of the angle in the QX,QY,QZ,QJX,QJY,QJZ, set_values, set_as_QX,...
functions to be consistent with the corrected multiplication. Also updated
the doc strings."""

"""Version 3 September 8, 2010 RLH
Backed out change to cnvrt in version 2."""

"""Version 2 September 3, 2010 RLH
Fixed sign error in quaternion multiplication functions.
Added quaternion __str__ method.
Modified cnvrt method to return a CelestialVector."""


# 4/9/2010 WMK
# Redefined the __init__ inputs
# Changed from a Vector internal representation to 3 scalers
# Fixed an error in cvt_att_Q_to_angles, was assuming an att2inertial Quaternion!
# Streamlined some of the functions
#  Version 1.0 August 3, 2010
#  Got rid of degrees trig functions.

#from math import *
import math
from .math_extensionsx import *
from . import rotationsx as rot

D2R = pi / 180.
R2D = 180. / pi
PI2 = 2. * pi


def unit_limit(x): return min(max(-1., x), 1.)


def QX(angle):
    """Creates rotation quaternion about X axis, assuming coordinate frame is rotated"""
    return Quaternion(rot.Vector(math.sin(-angle / 2.),
                                 0., 0.), math.cos(angle / 2.))


def QY(angle):
    """Creates rotation quaternion about Y axis, assuming coordinate frame is rotated"""
    return Quaternion(rot.Vector(
        0., math.sin(-angle / 2.), 0.), math.cos(angle / 2.))


def QZ(angle):
    """Creates rotation quaternion about Z axis, assuming coordinate frame is rotated"""
    return Quaternion(rot.Vector(
        0., 0., math.sin(-angle / 2.)), math.cos(angle / 2.))


def Qmake_a_point(V):
    """Creates a pure Q, i.e. defines a pointing not a rotation"""
    return Quaternion(V, 0.)


def cvt_pt_Q_to_V(Q):
    """Converts a pure (pointing) Q to a unit position Vector"""
    return rot.Vector(Q.q1, Q.q2, Q.q3)


# The following functions are dependent upon the spacecraft definitions
# and perhaps should be moved to that module

def Qmake_an_attitude(ra, dec, pa):
    """Creates a rotation Q, going from the inertial to the attitude"""
    return QX(-pa) * QY(-dec) * QZ(ra)


def Qmake_inertial2att(coord1, coord2, pa):
    """Creates a rotation Q, going from the inertial to the attitude"""
    return QX(-pa) * QY(-coord2) * QZ(coord1)


def Qmake_an_attitude_w_v2v3(ra, dec, pa, v2, v3):
    return QZ(-v2) * QY(v3) * QX(-pa) * QY(-dec) * QZ(ra)


def Qmake_inertial2att_full(ra, dec, pa, xa, ya, s, ta, v2a, v3a):
    """ Where
        ra - right ascension of target
        dec - declination of target
        pa - position angle of the +y-axis of the aperture coordinate system
        xa,ya - coordinates of the target in the aperture coordinate system
        s - parity of the x-axis of the aperture coordinate system (s = +/- 1)
        ta - position angle of the +y-axis of the aperture coordinate system at
             the origin of the aperture coordinate with respect to the +V3 axis.
        v2a,v3a - coordinates of the origin of the aperture coordinate system in
             the vehicle frame
        Orientation Matrix John C. Isaacs Sep. 16, 2010

    """
    return QZ(-v2a) * QY(v3a) * QX(ta) * QZ(-s * xa) * \
        QY(ya) * QX(-pa) * QY(-dec) * QZ(ra)


def cvt_inert2att_Q_to_angles(Q):
    """Creates a angle tuple from Q, assuming an inertial to attitude Q"""
    V1_body = rot.Vector(1., 0., 0.)
    V1_eci_pt = Q.inv_cnvrt(V1_body)
    coord1 = math.atan2(V1_eci_pt.y, V1_eci_pt.x)
    if coord1 < 0.:
        coord1 += PI2
    coord2 = math.asin(unit_limit(V1_eci_pt.z))

    V3_body = rot.Vector(0., 0., 1.)
    V3_eci_pt = Q.inv_cnvrt(V3_body)
    NP_eci = rot.Vector(0., 0., 1.)
    V_left = rot.cross(NP_eci, V1_eci_pt)
    V_left = V_left / V_left.length()
    NP_in_plane = rot.cross(V1_eci_pt, V_left)
    x = rot.dot(V3_eci_pt, NP_in_plane)
    y = rot.dot(V3_eci_pt, V_left)
    pa = math.atan2(y, x)
    if pa < 0.:
        pa += PI2
    return (coord1, coord2, pa)


def cvt_att_Q_to_angles2(Q):
    """This assumes the quaternion is inertial to S/C and the Euler angles
 are 321 mapped to ra,-dec,-pa"""
    q1 = Q.q1
    q2 = Q.q2
    q3 = Q.q3
    q4 = Q.q4
    ra = math.atan2(2. * (q1 * q2 - q3 * q4),
                    q1 * q1 - q2 * q2 - q3 * q3 + q4 * q4)
    if ra < 0.:
        ra += PI2
    arg = min(1., max(-1., 2. * (q1 * q3 + q2 * q4)))
    dec = math.asin(arg)
    pa = math.atan2(-2. * (q2 * q3 - q1 * q4), -q1 *
                    q1 - q2 * q2 + q3 * q3 + q4 * q4)
    if pa < 0.:
        pa += PI2
    return (ra, dec, pa)


def Qmake_body2inertial(coord1, coord2, V3pa):
    """Creates a rotation Q, going from the body frame to inertial"""
    return QZ(coord1) * QY(-coord2) * QX(-V3pa)


def Qmake_v2v3_2body(v2, v3):
    """Creates a rotation Q, going from v2 and v3 in the body frame to inertial"""
    return QY(v3) * QZ(-v2)


def Qmake_v2v3_2inertial(coord1, coord2, V3pa, v2, v3):
    """Creates a rotation Q, going from v2 and v3 in the body frame to inertial"""
    return QZ(coord1) * QY(-coord2) * QX(-V3pa) * QY(v3) * QZ(-v2)


def Qmake_aperture2inertial(
        coord1, coord2, APA, xoff, yoff, s, YapPA, V3ref, V2ref):
    """Creates a rotation Q, going from the target in aperture frame to body"""
    return QZ(coord1) * QY(-coord2) * QX(-APA) * QY(-yoff) * \
        QZ(s * xoff) * QX(YapPA) * QY(V3ref) * QZ(-V2ref)


def cvt_body2inertial_Q_to_c1c2pa_tuple(Q):
    """Creates a angle tuple from Q, assuming body frame to inertial Q and 321 rotation sequence"""
    # Conversion from Euler symmetric parameters to matrix elements and matrix elements to rotation angles
    # is given in Isaac's papers
    r11 = Q.q1 * Q.q1 - Q.q2 * Q.q2 - Q.q3 * Q.q3 + Q.q4 * Q.q4
    r21 = 2. * (Q.q1 * Q.q2 + Q.q3 * Q.q4)
    r31 = 2. * (Q.q1 * Q.q3 - Q.q2 * Q.q4)
    r32 = 2. * (Q.q2 * Q.q3 + Q.q1 * Q.q4)
    r33 = -Q.q1 * Q.q1 - Q.q2 * Q.q2 + Q.q3 * Q.q3 + Q.q4 * Q.q4
    coord1 = atan2(r21, r11)
    if coord1 < 0.:
        coord1 += PI2
    coord2 = asin2(r31)  # use "safe" version of sine
    pa = atan2(-r32, r33)
    if pa < 0.:
        pa += PI2
    return (coord1, coord2, pa)


def cvt_v2v3_using_body2inertial_Q_to_c1c2pa_tuple(Q, v2, v3):
    """Given Q and v2,v3 gives pos on sky and V3 PA """
    Vp_body = rot.Vector(0., 0., 0.)
    Vp_body.set_xyz_from_angs(v2, v3)
    Vp_eci_pt = Q.cnvrt(Vp_body)
    coord1 = atan2(Vp_eci_pt.y, Vp_eci_pt.x)
    if coord1 < 0.:
        coord1 += PI2
    coord2 = asin(unit_limit(Vp_eci_pt.z))

    V3_body = rot.Vector(0., 0., 1.)
    V3_eci_pt = Q.cnvrt(V3_body)
    NP_eci = rot.Vector(0., 0., 1.)
    V_left = cross(NP_eci, Vp_eci_pt)
    if V_left.length() > 0.:
        V_left = V_left / V_left.length()
    NP_in_plane = cross(Vp_eci_pt, V_left)
    x = dot(V3_eci_pt, NP_in_plane)
    y = dot(V3_eci_pt, V_left)
    pa = atan2(y, x)
    if pa < 0.:
        pa += PI2

    return (coord1, coord2, pa)


def cvt_c1c2_using_body2inertial_Q_to_v2v3pa_tuple(Q, coord1, coord2):
    """Given Q and a position, returns v2,v3,V3PA tuple """
    Vp_eci = rot.Vector(1., 0., 0.)
    Vp_eci.set_xyz_from_angs(coord1, coord2)
    Vp_body_pt = Q.inv_cnvrt(Vp_eci)
    v2 = atan2(Vp_body_pt.y, Vp_body_pt.x)
    v3 = asin(unit_limit(Vp_body_pt.z))
    V3_body = rot.Vector(0., 0., 1.)
    V3_eci_pt = self.cnvrt(V3_body)
    NP_eci = rot.Vector(0., 0., 1.)
    V_left = cross(NP_eci, Vp_eci)
    if V_left.length() > 0.:
        V_left = V_left / V_left.length()
    NP_in_plane = cross(Vp_eci, V_left)
    x = dot(V3_eci_pt, NP_in_plane)
    y = dot(V3_eci_pt, V_left)
    pa = atan2(y, x)
    if pa < 0.:
        pa += PI2
    return (v2, v3, pa)


############################################################


class Quaternion:
    """This representation is used by Wertz and Markley """

    def __init__(self, V, q4):
        """Quaternion constructor """
        self.q1 = V.x
        self.q2 = V.y
        self.q3 = V.z
        self.q4 = q4

    def __str__(self):
        """Returns a string representation of the quaternion."""

        return('Quaternion: q1: %.3f, q2: %.3f, q3: %.3f, q4: %.3f'
               % (self.q1, self.q2, self.q3, self.q4))

    def length(self):
        """Returns length of the Q """
        return math.sqrt(self.q1 * self.q1 + self.q2 *
                         self.q2 + self.q3 * self.q3 + self.q4 * self.q4)

    def normalize(self):
        """Returns a copy of the Q normalized """
        scale = self.length()
        return Quaternion(rot.Vector(self.q1 / scale, self.q2 /
                                     scale, self.q3 / scale), self.q4 / scale)

    def conjugate(self):
        """Returns a copy of the conjugated Q """
        return Quaternion(rot.Vector(-self.q1, -self.q2, -self.q3), self.q4)

    def __mul__(self, rs):
        """Defines Q*Q for quaternion multiplication """
        Q = Quaternion(rot.Vector(0., 0., 0.), 0.)
        #Q.V = rs.V * self.q4 + self.V * rs.q4 + cross(self.V,rs.V)
        Q.q1 = rs.q1 * self.q4 + self.q1 * rs.q4 + \
            (self.q2 * rs.q3 - self.q3 * rs.q2)
        Q.q2 = rs.q2 * self.q4 + self.q2 * rs.q4 + \
            (self.q3 * rs.q1 - self.q1 * rs.q3)
        Q.q3 = rs.q3 * self.q4 + self.q3 * rs.q4 + \
            (self.q1 * rs.q2 - self.q2 * rs.q1)
        Q.q4 = self.q4 * rs.q4 - \
            (self.q1 * rs.q1 + self.q2 * rs.q2 + self.q3 * rs.q3)
        return Q

    def cnvrt(self, V):
        """Rotates a vector from the starting frame to the ending frame defined by the Q """
        QV = Qmake_a_point(V)
        QV = self * QV * self.conjugate()
        return rot.Vector(QV.q1, QV.q2, QV.q3)

    def inv_cnvrt(self, V):
        """Rotates a vector from the ending frame to the starting frame defined by the Q"""
        QV = Qmake_a_point(V)
        QV = self.conjugate() * QV * self
        return rot.Vector(QV.q1, QV.q2, QV.q3)

    def set_values(self, V, angle):
        """Sets quaterion values using a direction vector and a rotation of the coordinate frame about it."""
        S = sin(-angle / 2.)
        self.q1 = V.x * S
        self.q2 = V.y * S
        self.q3 = V.z * S
        self.q4 = cos(angle / 2.)

    def set_as_QX(self, angle):
        """Sets quaterion in place like QX function"""
        self.q1 = sin(-angle / 2.)
        self.q2 = 0.
        self.q3 = 0.
        self.q4 = cos(angle / 2.)

    def set_as_QY(self, angle):
        """Sets quaterion in place like QY function"""
        self.q1 = 0.
        self.q2 = sin(-angle / 2.)
        self.q3 = 0.
        self.q4 = cos(angle / 2.)

    def set_as_QZ(self, angle):
        """Sets quaterion in place like QZ function"""
        self.q1 = 0.
        self.q2 = 0.
        self.q3 = sin(-angle / 2.)
        self.q4 = cos(angle / 2.)

    def set_as_mult(self, QQ1, QQ2):
        """Sets self as QQ1*QQ2 in place for quaternion multiplication """
        self.q1 = QQ2.q1 * QQ1.q4 + QQ1.q1 * QQ2.q4 + \
            (QQ1.q2 * QQ2.q3 - QQ1.q3 * QQ2.q2)
        self.q2 = QQ2.q2 * QQ1.q4 + QQ1.q2 * QQ2.q4 + \
            (QQ1.q3 * QQ2.q1 - QQ1.q1 * QQ2.q3)
        self.q3 = QQ2.q3 * QQ1.q4 + QQ1.q3 * QQ2.q4 + \
            (QQ1.q1 * QQ2.q2 - QQ1.q2 * QQ2.q1)
        self.q4 = QQ1.q4 * QQ2.q4 - \
            (QQ1.q1 * QQ2.q1 + QQ1.q2 * QQ2.q2 + QQ1.q3 * QQ2.q3)

    def set_as_point(self, V):
        self.q1 = V.x
        self.q2 = V.y
        self.q3 = V.z
        self.q4 = 0.

    def set_equal(self, Q):
        """Assigns values from other Q to this one. """
        self.q1 = Q.q1
        self.q2 = Q.q2
        self.q3 = Q.q3
        self.q4 = Q.q4

    def set_as_conjugate(self):
        """Assigns conjugate values in place. """
        self.q1 *= -1.
        self.q2 *= -1.
        self.q3 *= -1.
